/***********************************************************************/
/*                                                                     */
/*  FILE        :backup.c                                              */
/*  DESCRIPTION :backup with DataFlash                                 */
/*  CPU TYPE    :RX210                                                 */
/*                                                                     */
/*  last modefied : Oct 02, 2014                                       */
/*  for CQ-HF panel  AOR  by KOBAYASHI Ikuo                            */
/*                                                                     */
/***********************************************************************/

//#include "variation.h"
//#include <machine.h>
#include <string.h>
#include <stdio.h>
//#include "iodefine.h"
#include "display.h"
#include "vfo_param.h"
#include "r_flash_api_rx_if.h"

#ifdef __cplusplus
extern "C" {
#endif

void flash_init(void);

void flash_test01(void);

void backup_main(void);
static void backup_exec_Init(void);
static void backup_exec_CheckReadTop(void);
static void backup_exec_CheckRead(void);
static void backup_exec_CheckCrc(void);
static void backup_exec_CheckValid(void);
static void backup_exec_CheckOlder(void);
static void backup_exec_CheckReadBottom(void);
static void backup_exec_CheckPageFix(void);
static void backup_exec_ReadPageRead(void);
static void backup_exec_ReadFlash2Ram(void);
static void backup_exec_ReadInitialData(void);
static void backup_exec_Idoling(void);
static void backup_exec_WriteStart(void);
static void backup_exec_WriteRam2Flash(void);
static void backup_exec_WriteCrc(void);
static void backup_exec_WriteCompare(void);
static void backup_exec_WriteOlderBlank(void);
static void backup_exec_WriteOlder(void);
static void backup_exec_WriteErase(void);
static void backup_exec_WriteLoopStart(void);
static void backup_exec_WriteLoopTop(void);
static void backup_exec_WriteWrite(void);
static void backup_exec_WriteLoopBottom(void);
static void backup_exec_EraseAll(void);



void backup_write_req(unsigned char v);
void backup_read_req(void);
void backup_EraceAll_req(void);

void backup_MemToBuf(unsigned char *bufptr);
void backup_BufToMem(unsigned char *bufptr);
void backup_MemInit(unsigned char *bufptr);
void backup_BufAddCRC(unsigned char *bufptr);
unsigned char backup_BufCheckCRC(unsigned char *bufptr);
unsigned char backup_BufCheckValid(unsigned char *bufptr);

void backup_timer(void);
void backup_timer_start(unsigned short t);
void backup_timer_stop(void);


#ifdef __cplusplus
}
#endif

#define BACKUP_TOP_ADRS (0x00100000)
#define	BACKUP_PAGE_MAX	(8)
#define	BACKUP_BUF_SIZE	(256)
#define	BACKUP_BUF_OLDER_SIZE	(8)
#define	BACKUP_BUF_OLDER_POS	(BACKUP_BUF_SIZE - BACKUP_BUF_OLDER_SIZE)
#define	BACKUP_BUF_DATACRC_SIZE	(BACKUP_BUF_SIZE - BACKUP_BUF_OLDER_SIZE)
#define	BACKUP_BUF_CRC_SIZE	(2)
#define	BACKUP_BUF_CRC_POS	(BACKUP_OLDER_POS - BACKUP_BUF_CRC_SIZE)
#define	BACKUP_BUF_DATA_SIZE	(BACKUP_BUF_DATACRC_SIZE - BACKUP_BUF_CRC_SIZE)

#define	BACKUP_OLDER_MIN	(0x10)
#define	BACKUP_OLDER_MAX	(0xe0)

#define	BACKUP_WAIT_DIAL	(60000)
#define	BACKUP_WAIT_SW	(10000)
#define	BACKUP_WAIT_MENU	(5000)
#define	BACKUP_WAIT_NOW	(1)


typedef enum
{ 
    BACKUP_INIT, 
    BACKUP_CHECK_READ_TOP,
	BACKUP_CHECK_READ, 
    BACKUP_CHECK_CRC, 
    BACKUP_CHECK_VALID, 
    BACKUP_CHECK_OLDER,
	BACKUP_CHECK_READ_BOTTOM,
	BACKUP_CHECK_PAGEFIX,
	BACKUP_READ_PAGEREAD,
	BACKUP_READ_FLASH2RAM,
	BACKUP_READ_INITIALDATA,
	BACKUP_IDOLING,
	BACKUP_WRITE_START,
	BACKUP_WRITE_RAM2FLASH,
	BACKUP_WRITE_CRC,
	BACKUP_WRITE_COMPARE,
	BACKUP_WRITE_OLDER_BLANK,
	BACKUP_WRITE_OLDER,
	BACKUP_WRITE_ERASE,
	BACKUP_WRITE_LOOP_START,
	BACKUP_WRITE_LOOP_TOP,
	BACKUP_WRITE_WRITE,
	BACKUP_WRITE_LOOP_BOTTOM,
	BACKUP_ERASE_ALL,
	
	
} flash_states_t;


#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO) || defined(FLASH_API_RX_CFG_ROM_BGO)
/* Used to tell application whether previous flash operation has finished */
static uint8_t flash_is_ready;
#endif
#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
/* Holds the outcome of the blank check operation */
static uint8_t blank_check_result;
#endif

flash_states_t backup_state;

unsigned char *flash_adrs;
unsigned short flash_crc,flash_index;

unsigned short backup_timer_count;
unsigned char backup_timer_flag;
unsigned char backup_req_flag;
unsigned char backup_req_flag2; 
unsigned char backup_valid_page, backup_prev_older,backup_next_older;
unsigned char backup_page,backup_info_flag;
unsigned char backup_older_val[BACKUP_PAGE_MAX];
unsigned char flash_buf[256],backup_buf[256];


//unsigned char flash_read_buf[128];
//unsigned char flash_write_buf[128];


extern struct VfoParamStruct CurrentVfo,VfoArray[2];
extern long RitOffset;
extern unsigned long IFFreq;
extern unsigned short CWPitch;
extern short ShiftFreq;
extern unsigned char VfoNo;
extern unsigned char SQOpened;
//extern unsigned char FreqCursorPos;
extern unsigned char VolVal,SQVal;
extern unsigned char RFGain,MicGain;
extern unsigned char LevelSQ_Hysteresis;
extern unsigned char CurrentVoiceSQ_Kando,CurrentVoiceSQ_Delay,CurrentVoiceSQ_Enable;
extern unsigned char DeNoise,AutoNotch,NoiseBlanker;
extern unsigned char AfcVal;
extern unsigned char SemiBreakInHoldTime;
extern unsigned char AMModulationDepth,TXPower;
extern unsigned char TXMeterType;
extern unsigned char SuperHeteroFlag;
extern unsigned char TestModeFlag,TestToneFlag;
extern unsigned char IFShiftEnable;
extern unsigned char CWCarrier,CWSideTone;
extern unsigned char BeepVol;

extern void lcd_disp_SetCursor_Menu(void);

const unsigned char BitMask8bit[8] = {
	0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
};

//==========================================
//  flash_init
//		Flashg߂̏ 
//==========================================
void flash_init(void)
{
unsigned char r;
	R_FlashCodeCopy();
    R_FlashDataAreaAccess(0xffff, 0xffff);
}

#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)

#pragma section FRAM
/******************************************************************************
* Function Name: FlashBlankCheckDone
* Description  : Callback function for when blank check operation has finished
* Arguments    : result - FLASH_BLANK if block was blank
*                         FLASH_NOT_BLANK if block was not blank
* Return Value : none
******************************************************************************/
void FlashBlankCheckDone(uint8_t result)
{
    /* Copy result of blank check */
    blank_check_result = result;
    
    /* Signal flash operation is done */
    flash_is_ready = true;
}
/******************************************************************************
End of function  FlashBlankCheckDone
******************************************************************************/

#endif /* defined(FLASH_API_RX_CFG_DATA_FLASH_BGO) */

#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO) || defined(FLASH_API_RX_CFG_ROM_BGO)

#pragma section FRAM
/******************************************************************************
* Function Name: FlashEraseDone
* Description  : Callback function for when blank check operation has finished
* Arguments    : none
* Return Value : none
******************************************************************************/
void FlashEraseDone(void)
{
    /* Signal flash operation is done */
    flash_is_ready = true;
}
/******************************************************************************
End of function  FlashEraseDone
******************************************************************************/

/******************************************************************************
* Function Name: FlashWriteDone
* Description  : Callback function for when blank check operation has finished
* Arguments    : none
* Return Value : none
******************************************************************************/
void FlashWriteDone(void)
{
    /* Signal flash operation is done */
    flash_is_ready = true;
}
/******************************************************************************
End of function  FlashWriteDone
******************************************************************************/

/******************************************************************************
* Function Name: FlashError
* Description  : Callback function for when blank check operation has finished
* Arguments    : none
* Return Value : none
******************************************************************************/
void FlashError(void)
{
    /* Alert program or user that flash error has occurred */
    while(1)
    {
nop();
nop();
        /* Error in demo. */
    }
}
/******************************************************************************
End of function  FlashError
******************************************************************************/
#pragma section

#endif /* defined(FLASH_API_RX_CFG_DATA_FLASH_BGO) || defined(FLASH_API_RX_CFG_ROM_BGO) */

//==========================================
//  backup_main
//      backup control main
//==========================================
void backup_main(void)
{
	switch(backup_state){
		case BACKUP_INIT:	// 	
			backup_exec_Init();  break;
		case BACKUP_CHECK_READ_TOP:	// Sy[W`FbÑ[vgbv
			backup_exec_CheckReadTop();  break;
		case BACKUP_CHECK_READ:	// 1pageRead
			backup_exec_CheckRead();  break;
		case BACKUP_CHECK_CRC:	// CRC check
			backup_exec_CheckCrc();  break;
		case BACKUP_CHECK_VALID:	// ValidData check
			backup_exec_CheckValid();  break;
		case BACKUP_CHECK_OLDER:	// OlderVal check
			backup_exec_CheckOlder();  break;
		case BACKUP_CHECK_READ_BOTTOM:	// Sy[W`FbÑ[vGh
			backup_exec_CheckReadBottom();  break;
		case BACKUP_CHECK_PAGEFIX:	// ǂݏoy[Wm
			backup_exec_CheckPageFix();  break;
		case BACKUP_READ_PAGEREAD:	// 1pageǂݍ
			backup_exec_ReadPageRead();  break;
		case BACKUP_READ_FLASH2RAM:	// [NRAMւ̓WJ
			backup_exec_ReadFlash2Ram();  break;
		case BACKUP_READ_INITIALDATA:	// lݒ
			backup_exec_ReadInitialData();  break;
		case BACKUP_IDOLING:	// ҂
			backup_exec_Idoling();  break;
		case BACKUP_WRITE_START:	// ݊Jn
			backup_exec_WriteStart();  break;
		case BACKUP_WRITE_RAM2FLASH:	// bufpack
			backup_exec_WriteRam2Flash();  break;
		case BACKUP_WRITE_CRC:	// CRCt
			backup_exec_WriteCrc();  break;
		case BACKUP_WRITE_COMPARE:	// ύX`FbN
			backup_exec_WriteCompare();  break;
		case BACKUP_WRITE_OLDER_BLANK:	// olderBrank check
			backup_exec_WriteOlderBlank();  break;
		case BACKUP_WRITE_OLDER:	// older
			backup_exec_WriteOlder();  break;
		case BACKUP_WRITE_ERASE:	// ݗ\n̏
			backup_exec_WriteErase();
		case BACKUP_WRITE_LOOP_START:	// ݃[vJn
			backup_exec_WriteLoopStart();  break;
		case BACKUP_WRITE_LOOP_TOP:	// ݃[v擪
			backup_exec_WriteLoopTop();  break;
		case BACKUP_WRITE_WRITE:	// ݃[v
			backup_exec_WriteWrite();  break;
		case BACKUP_WRITE_LOOP_BOTTOM:	// ݃[vbottom
			backup_exec_WriteLoopBottom();  break;
		case BACKUP_ERASE_ALL:	// DataFlashS
			backup_exec_EraseAll();
	}
}


//==========================================
//  backup_exec_Init
//      state BACKUP_INIT   
//==========================================
static void backup_exec_Init(void)
{
unsigned char i;
	backup_timer_stop();
	backup_page = backup_valid_page = 0;
	backup_info_flag = 0;
	for(i=0; i<BACKUP_PAGE_MAX; i++){
		backup_older_val[i] = 0;
	}
	backup_state = BACKUP_CHECK_READ_TOP;
}

//==========================================
//  backup_exec_CheckReadTop
//      state BACKUP_CHECK_READ_TOP  [vgbv 
//==========================================
static void backup_exec_CheckReadTop(void)
{
	if( backup_page >= BACKUP_PAGE_MAX){
		backup_state = BACKUP_CHECK_PAGEFIX;
	}else{
		backup_state = BACKUP_CHECK_READ;
	}
}

//==========================================
//  backup_exec_CheckRead
//      state BACKUP_CHECK_READ  1pageRead 
//==========================================
static void backup_exec_CheckRead(void)
{
unsigned char *f_ad;
unsigned short i;
	f_ad = (unsigned char *)(backup_page * BACKUP_BUF_SIZE + BACKUP_TOP_ADRS);
	for(i=0; i<BACKUP_BUF_SIZE; i++){
		flash_buf[i] = *f_ad++;
	}
	backup_state = BACKUP_CHECK_CRC;
}

//==========================================
//  backup_exec_CheckCrc
//      state BACKUP_CHECK_CRC  CRC check 
//==========================================
static void backup_exec_CheckCrc(void)
{
unsigned char r;
	r = backup_BufCheckCRC(flash_buf);
	if(r){
		backup_state = BACKUP_CHECK_VALID;
	}else{
		backup_info_flag &= ~BitMask8bit[backup_page];
		backup_older_val[backup_page] = 0x00;
		backup_state = BACKUP_CHECK_READ_BOTTOM;
	}
}

//==========================================
//  backup_exec_CheckValid
//      state BACKUP_CHECK_VALID  ValidData check 
//==========================================
static void backup_exec_CheckValid(void)
{
unsigned char r;
	r = backup_BufCheckValid(flash_buf);
	if(r){
		backup_info_flag |= BitMask8bit[backup_page];
		backup_state = BACKUP_CHECK_OLDER;
	}else{
		backup_info_flag &= ~BitMask8bit[backup_page];
		backup_older_val[backup_page] = 0x00;
		backup_state = BACKUP_CHECK_READ_BOTTOM;
	}
}

//==========================================
//  backup_exec_CheckOlder
//      state BACKUP_CHECK_OLDER  OlderVal check 
//==========================================
static void backup_exec_CheckOlder(void)
{
unsigned char *f_ad;
unsigned char r;
	f_ad = (unsigned char *)(BACKUP_BUF_OLDER_POS + (backup_page * BACKUP_BUF_SIZE) + BACKUP_TOP_ADRS);
	r = R_FlashDataAreaBlankCheck(f_ad, BLANK_CHECK_2_BYTE);
	if(r==FLASH_BLANK){
		backup_older_val[backup_page] = 0xff;
	}else{
		backup_older_val[backup_page] = flash_buf[BACKUP_BUF_OLDER_POS];
	}
	backup_state = BACKUP_CHECK_READ_BOTTOM;
}

//==========================================
//  backup_exec_CheckReadBottom
//      state BACKUP_CHECK_READ_BOTTOM  [vGh 
//==========================================
static void backup_exec_CheckReadBottom(void)
{
	backup_page++;
	if( backup_page >= BACKUP_PAGE_MAX){
		backup_state = BACKUP_CHECK_PAGEFIX;
	}else{
		backup_state = BACKUP_CHECK_READ_TOP;
	}
}

//==========================================
//  backup_exec_CheckPageFix
//      state BACKUP_CHECK_PAGE_FIX  ǂݏoy[Wm 
//==========================================
static void backup_exec_CheckPageFix(void)
{
unsigned char i,b;
unsigned char vmax,vmax2,vmax3;
unsigned char imax,imax2,imax3;
	if(backup_info_flag == 0x00){
		backup_valid_page = 0;
		backup_prev_older = BACKUP_OLDER_MIN;
		backup_state = BACKUP_READ_INITIALDATA;
		return;
	}
	//
	vmax = vmax2 = vmax3 = 0;
	imax = imax2 = imax3 = BACKUP_PAGE_MAX + 3;
	for(i=0; i<BACKUP_PAGE_MAX; i++){
		if((backup_info_flag & BitMask8bit[i]) != 0x00){
			b = backup_older_val[i];
			if(b > vmax){
				vmax = b;
				imax = i;
			}
			if(b != 0xff){
				if(b > vmax2){
					vmax2 = b;
					imax2 = i;
				}
			}
		}
	}
	
	if(vmax2 > (BACKUP_OLDER_MAX - 0x10) ){
		for(i=0; i<BACKUP_PAGE_MAX; i++){
			if((backup_info_flag & BitMask8bit[i]) != 0x00){
				b = backup_older_val[i];
				if(b <= (BACKUP_OLDER_MIN + 0x10) ){
					if(b > vmax3){
						vmax3 = b;
						imax3 = i;
					}
				}
			}
		}
		if(imax3 < BACKUP_PAGE_MAX){
			vmax2 = vmax3;
			imax2 = imax3;
		}
	}
	
	if(imax < BACKUP_PAGE_MAX){
		backup_valid_page = imax;
		if(vmax == 0xff){
			backup_prev_older = vmax2;
		}else{
			backup_prev_older = vmax;
		}
		if(backup_prev_older < BACKUP_OLDER_MIN){
			backup_prev_older = BACKUP_OLDER_MIN;
		}
		backup_state = BACKUP_READ_PAGEREAD;
	}else{
		backup_valid_page = 0;
		backup_prev_older = BACKUP_OLDER_MIN;
		backup_state = BACKUP_READ_INITIALDATA;
	}
}

//==========================================
//  backup_exec_ReadPageRead
//      state BACKUP_READ_PAGEREAD  1pageRead 
//==========================================
static void backup_exec_ReadPageRead(void)
{
unsigned char *f_ad;
unsigned short i;
	f_ad = (unsigned char *)(backup_valid_page * BACKUP_BUF_SIZE + BACKUP_TOP_ADRS);
	for(i=0; i<BACKUP_BUF_SIZE; i++){
		flash_buf[i] = *f_ad++;
	}
	backup_state = BACKUP_READ_FLASH2RAM;
}

//==========================================
//  backup_exec_ReadFlash2Ram
//      state BACKUP_READ_FLASH2RAM  RAMɓWJ 
//==========================================
static void backup_exec_ReadFlash2Ram(void)
{
extern void lcd_disp_AllRefresh(void);
extern unsigned char MainMode;
	backup_BufToMem(flash_buf);
	backup_state = BACKUP_IDOLING;
	
	MainMode = MAIN_MODE_VFO;
	lcd_disp_AllRefresh();
}

//==========================================
//  backup_exec_ReadInitialData
//      state BACKUP_READ_INITIALDATA  lݒ 
//==========================================
static void backup_exec_ReadInitialData(void)
{
	memset(flash_buf,0x00,BACKUP_BUF_SIZE);
	backup_MemInit(flash_buf);
	backup_state = BACKUP_READ_FLASH2RAM;
	
backup_timer_start(100);

}

//==========================================
//  backup_exec_Idoling
//      state BACKUP_IDOLING  ҂ 
//==========================================
static void backup_exec_Idoling(void)
{
	if(backup_timer_flag == 0xff){
		backup_timer_stop();
		backup_state = BACKUP_WRITE_START;
		return;
	}
	if((backup_req_flag2 & 0x01) != 0x00){
		backup_req_flag2 &= ~0x01;
		backup_state = BACKUP_ERASE_ALL;
		return;
	}
	if((backup_req_flag2 & 0x02) != 0x00){
		backup_req_flag2 &= ~0x02;
		backup_state = BACKUP_INIT;
		return;
	}
}

//==========================================
//  backup_exec_WriteStart
//      state BACKUP_WRITE_START  ݊Jn 
//==========================================
static void backup_exec_WriteStart(void)
{
	backup_page = backup_valid_page;
	backup_page++;
	if(backup_page >= BACKUP_PAGE_MAX){
		backup_page = 0;
	}
	backup_state = BACKUP_WRITE_RAM2FLASH;
}

//==========================================
//  backup_exec_WriteRam2Flash
//      state BACKUP_WRITE_RAM2FLASH  bufpack 
//==========================================
static void backup_exec_WriteRam2Flash(void)
{
	memset(backup_buf, 0x00, BACKUP_BUF_SIZE);
	backup_MemToBuf(backup_buf);
	backup_state = BACKUP_WRITE_CRC;
}

//==========================================
//  backup_exec_WriteCrc
//      state BACKUP_WRITE_CRC  CRC vZǉ 
//==========================================
static void backup_exec_WriteCrc(void)
{
	backup_BufAddCRC(backup_buf);
	backup_state = BACKUP_WRITE_COMPARE;
}

//==========================================
//  backup_exec_WriteCompare
//      state BACKUP_WRITE_COMPARE  ύX`FbN 
//==========================================
static void backup_exec_WriteCompare(void)
{
unsigned short i;
unsigned char f;
//	f = 0;
//	for(i=0; i<BACKUP_BUF_DATACRC_SIZE; i++){
//		if(backup_buf[i] != flash_buf[i]){
//			f++;
//			break;
//		}
//	}
//	if(f == 0){
//		backup_state = BACKUP_WRITE_IDOLING;
//		return;
//	}
	//
	for(i=0; i<BACKUP_BUF_DATACRC_SIZE; i++){
		flash_buf[i] = backup_buf[i];
	}
	backup_next_older = backup_prev_older;
	backup_next_older++;
	if(backup_next_older >= BACKUP_OLDER_MAX){
		backup_next_older = BACKUP_OLDER_MIN;
	}
	flash_buf[i++] = backup_next_older;
	for( ; i<BACKUP_BUF_SIZE; i++){
		flash_buf[i] = 0;
	}
	backup_state = BACKUP_WRITE_OLDER_BLANK;
	
	lcd_locate(5,1,0);
	lcd_putchar(0xeb);
	lcd_disp_SetCursor_Menu();
}

//==========================================
//  backup_exec_WriteOlderBlank
//      state BACKUP_WRITE_OLDER_BLANK  OlderVal brank check 
//==========================================
static void backup_exec_WriteOlderBlank(void)
{
unsigned char *f_ad;
unsigned char r;
	f_ad = (unsigned char *)(BACKUP_BUF_OLDER_POS + (backup_valid_page * BACKUP_BUF_SIZE) + BACKUP_TOP_ADRS);
	r = R_FlashDataAreaBlankCheck(f_ad, BLANK_CHECK_2_BYTE);
	if(r==FLASH_BLANK){
		backup_state = BACKUP_WRITE_OLDER;
	}else{
		backup_state = BACKUP_WRITE_ERASE;
	}
}

//==========================================
//  backup_exec_WriteOlder
//      state BACKUP_WRITE_OLDER  OlderVal write 
//==========================================
static void backup_exec_WriteOlder(void)
{
unsigned char *f_ad;
unsigned char r;
	f_ad = (unsigned char *)(BACKUP_BUF_OLDER_POS + (backup_valid_page * BACKUP_BUF_SIZE) + BACKUP_TOP_ADRS);
	r = R_FlashWrite(f_ad, &flash_buf[BACKUP_BUF_OLDER_POS], 8);
	if(r==FLASH_SUCCESS){
		backup_state = BACKUP_WRITE_ERASE;
	}else{
		backup_state = BACKUP_IDOLING;
lcd_locate(0,1,0);
lcd_print("Older W.Err");
lcd_disp_SetCursor_Menu();
	}
}

//==========================================
//  backup_exec_WriteErase
//      state BACKUP_WRITE_ERASE  ݗ\n 
//==========================================
static void backup_exec_WriteErase(void)
{
unsigned char *f_ad;
unsigned char r;
	f_ad = (unsigned char *)(0 + (backup_page * BACKUP_BUF_SIZE) + BACKUP_TOP_ADRS );
	r = R_FlashEraseRange(f_ad, BACKUP_BUF_SIZE);
	if(r==FLASH_SUCCESS){
		backup_state = BACKUP_WRITE_LOOP_START;
	}else{
		backup_state = BACKUP_IDOLING;
lcd_locate(0,1,0);
lcd_print("Erace Err");
lcd_disp_SetCursor_Menu();
	}
}

//==========================================
//  backup_exec_WriteLoopStart
//      state BACKUP_WRITE_LOOP_START  ݃[vJn 
//==========================================
static void backup_exec_WriteLoopStart(void)
{
unsigned char r;
	flash_adrs = (unsigned char *)( (backup_page * BACKUP_BUF_SIZE) + BACKUP_TOP_ADRS );
	flash_index = 0;
	backup_state = BACKUP_WRITE_LOOP_TOP;
}

//==========================================
//  backup_exec_WriteLoopTop
//      state BACKUP_WRITE_LOOP_TOP  ݃[v擪 
//==========================================
static void backup_exec_WriteLoopTop(void)
{
	backup_state = BACKUP_WRITE_WRITE;
}

//==========================================
//  backup_exec_WriteWrite
//      state BACKUP_WRITE_WRITE  ubN 
//==========================================
static void backup_exec_WriteWrite(void)
{
unsigned char r;
	r = R_FlashWrite(flash_adrs, &flash_buf[flash_index], 8);
	if(r==FLASH_SUCCESS){
		backup_state = BACKUP_WRITE_LOOP_BOTTOM;
	}else{
		backup_state = BACKUP_IDOLING;
lcd_locate(0,1,0);
lcd_print("W.Err");
lcd_disp_SetCursor_Menu();
	}
}

//==========================================
//  backup_exec_WriteLoopBottom
//      state BACKUP_WRITE_LOOP_BOTTOM  ݃[vŌ 
//==========================================
static void backup_exec_WriteLoopBottom(void)
{
	flash_adrs += 8;
	flash_index += 8;
	if(flash_index < BACKUP_BUF_DATACRC_SIZE){
		backup_state = BACKUP_WRITE_LOOP_TOP;
	}else{
		backup_valid_page = backup_page;
		backup_prev_older = backup_next_older;
		backup_state = BACKUP_IDOLING;
		
		lcd_locate(5,1,0);
		lcd_putchar(0x20);
		lcd_disp_SetCursor_Menu();
	}
}

//==========================================
//  backup_exec_EraseAll
//      state BACKUP_ERASE_ALL  DataFlashS 
//==========================================
static void backup_exec_EraseAll(void)
{
unsigned char *f_ad;
unsigned char r;
	f_ad = BACKUP_TOP_ADRS ;
	r = R_FlashEraseRange(f_ad, BACKUP_BUF_SIZE * BACKUP_PAGE_MAX);
	backup_state = BACKUP_IDOLING;
}


//==========================================
//  backup_write_req
//      backup ݊Jnv
//		in:v   =0:Dial쎞  =1:SW쎞  =2:j[ݒ莞  =3:
//==========================================
void backup_write_req(unsigned char v)
{
unsigned short w;
	switch(v){
		case 1: // SW
			w = BACKUP_WAIT_SW;
			break;
		case 2: // MENU
			w = BACKUP_WAIT_MENU;
			break;
		case 3: // NOW
			w = BACKUP_WAIT_NOW;
			break;
		case 0:	// Dial
		default:
			w = BACKUP_WAIT_DIAL;
			break;
	}
	backup_timer_start(w);
}

//==========================================
//  backup_EraseAll_req
//      backup  DataFlashSv
//==========================================
void backup_EraseAll_req(void)
{
	backup_req_flag2 |= 0x01;
}

//==========================================
//  backup_read_req
//      backup  DataFlashǂݍݗv
//==========================================
void backup_read_req(void)
{
	backup_req_flag2 |= 0x02;
}





//==========================================
//  backup_MemToBuf
//      WorkMemory -> FlashBuf packing
//==========================================
void backup_MemToBuf(unsigned char *bufptr)
{
unsigned char *p;
struct VfoParamStruct *vfop;
unsigned long *lp;
unsigned short *sp;
	p = memset(bufptr, 0x00, BACKUP_BUF_DATA_SIZE);
	p = bufptr;
	
	// CurrentVfo	// +0
	vfop = (struct VfoParamStruct *)p;
	*vfop++ = CurrentVfo;
	// VfoA		// +3c
	*vfop++ = VfoArray[0];
	// VfoB		// +78
	*vfop++ = VfoArray[1];
	lp = (unsigned long *)vfop;
	// RitOffset	// +b4
	*lp++ = RitOffset;
	// IFFreq	// +b8
	*lp++ = IFFreq;
	sp = (unsigned short *)lp;
	// CWPitch	// +bc
	*sp++ = CWPitch;
	// ShiftFreq	// +be
	*sp++ = ShiftFreq;
	p = (unsigned char *)sp;
	
	// VfoNo	// +c0
	*p++ = VfoNo;
	// RFGain	// +c1
	*p++ = RFGain;
	// MicGain	// +c2
	*p++ = MicGain;
	// LevelSQ_Hysteresis	// +c3
	*p++ = LevelSQ_Hysteresis;
	// CurrentVoiceSQ_Kando	// +c4
	*p++ = CurrentVoiceSQ_Kando;
	// CurrentVoiceSQ_Delay	// +c5
	*p++ = CurrentVoiceSQ_Delay;
	// CurrentVoiceSQ_Enable	// +c6
	*p++ = CurrentVoiceSQ_Enable;
	// DeNoise	// +c7
	*p++ = DeNoise;
	// AutoNotch	// +c8
	*p++ = AutoNotch;
	// NoiseBlanker	// +c9
	*p++ = NoiseBlanker;
	// AfcVal	// +ca
	*p++ = AfcVal;
	// SemiBreakInHoldTime	// +cb
	*p++ = SemiBreakInHoldTime;
	// AMModulationDepth	// +cc
	*p++ = AMModulationDepth;
	// TXPower	// +cd
	*p++ = TXPower;
	// TXMeterType	// +ce
	*p++ = TXMeterType;
	// SuperHeteroFlag	// +cf
	*p++ = SuperHeteroFlag;
	// TestModeFlag		// +d0
	*p++ = TestModeFlag;
	// TestToneFlag		// +d1
	*p++ = TestToneFlag;
	// IFShiftEnable	// +d2
	*p++ = IFShiftEnable;
	// CWCarrier	// +d3
	*p++ = CWCarrier;
	// CWSideTone	// +d4
	*p++ = CWSideTone;
	// BeepVol	// +d5
	*p++ = BeepVol;
	// VolVal	// +d6
	*p++ = VolVal;
	// SQVal	// +d7
	*p++ = SQVal;

}

//==========================================
//  backup_BufToMem
//      FlashBuf -> WorkMemory  unpacking
//==========================================
void backup_BufToMem(unsigned char *bufptr)
{
unsigned char *p;
struct VfoParamStruct *vfop;
unsigned long *lp;
unsigned short *sp;
	p = bufptr;
	
	// CurrentVfo	// +0
	vfop = (struct VfoParamStruct *)p;
	CurrentVfo = *vfop++;
	// VfoA		// +3c
	VfoArray[0] = *vfop++;
	// VfoB		// +78
	VfoArray[1] = *vfop++;
	lp = (unsigned long *)vfop;
	// RitOffset	// +b4
	RitOffset = *lp++;
	// IFFreq	// +b8
	IFFreq = *lp++;
	sp = (unsigned short *)lp;
	// CWPitch	// +bc
	CWPitch = *sp++;
	// ShiftFreq	// +be
	ShiftFreq = *sp++;
	p = (unsigned char *)sp;
	
	// VfoNo	// +c0
	VfoNo = *p++;
	// RFGain	// +c1
	RFGain = *p++;
	// MicGain	// +c2
	MicGain = *p++;
	// LevelSQ_Hysteresis	// +c3
	LevelSQ_Hysteresis = *p++;
	// CurrentVoiceSQ_Kando	// +c4
	CurrentVoiceSQ_Kando = *p++;
	// CurrentVoiceSQ_Delay	// +c5
	CurrentVoiceSQ_Delay = *p++;
	// CurrentVoiceSQ_Enable	// +c6
	CurrentVoiceSQ_Enable = *p++;
	// DeNoise	// +c7
	DeNoise = *p++;
	// AutoNotch	// +c8
	AutoNotch = *p++;
	// NoiseBlanker	// +c9
	NoiseBlanker = *p++;
	// AfcVal	// +ca
	AfcVal = *p++;
	// SemiBreakInHoldTime	// +cb
	SemiBreakInHoldTime = *p++;
	// AMModulationDepth	// +cc
	AMModulationDepth = *p++;
	// TXPower	// +cd
	TXPower = *p++;
	// TXMeterType	// +ce
	TXMeterType = *p++;
	// SuperHeteroFlag	// +cf
	SuperHeteroFlag = *p++;
	// TestModeFlag		// +d0
	TestModeFlag = *p++;
	// TestToneFlag		// +d1
	TestToneFlag = *p++;
	// IFShiftEnable	// +d2
	IFShiftEnable = *p++;
	// CWCarrier	// +d3
	CWCarrier = *p++;
	// CWSideTone	// +d4
	CWSideTone = *p++;
	// BeepVol	// +d5
	BeepVol = *p++;
	// VolVal	// +d6
	VolVal = *p++;
	// SQVal	// +d7
	SQVal = *p++;

}

//==========================================
//  backup_MemInit
//      WorkMemory initialise
//==========================================
void backup_MemInit(unsigned char *bufptr)
{
	
	// CurrentVfo	// +0
	//
	// VfoA		// +3c
	VfoArray[0].CurrentFreq = 7000000L+3000L;
	VfoArray[0].BandFreq[0] = 1810000L+3000L;
	VfoArray[0].BandFreq[1] = 3500000L+3000L;
	VfoArray[0].BandFreq[2] = 3702000L+3000L;
	VfoArray[0].BandFreq[3] = 7000000L+3000L;
	VfoArray[0].BandFreq[4] = 10100000L;
	VfoArray[0].BandFreq[5] = 14000000L;
	VfoArray[0].BandFreq[6] = 18068000L;
	VfoArray[0].BandFreq[7] = 21000000L;
	VfoArray[0].BandFreq[8] = 24890000L;
	VfoArray[0].BandFreq[9] = 28000000L;
	VfoArray[0].BandFreq[10] = 50000000L;
	VfoArray[0].BandFreq[11] = 76100000L;
	VfoArray[0].Mode = MODE_ITEM_LSB;	// LSB
	VfoArray[0].IFBandWidth = 3 + FILTER_ITEM_NOT_WFM;
	VfoArray[0].AGC = 2;
	VfoArray[0].BandNo = 3;
	VfoArray[0].Flag.BIT.RfAmp = 1;
	// VfoB		// +78
	VfoArray[1].CurrentFreq = 29000000L;
	VfoArray[1].BandFreq[0] = 1810000L+3000L;
	VfoArray[1].BandFreq[1] = 3500000L+3000L;
	VfoArray[1].BandFreq[2] = 3702000L+3000L;
	VfoArray[1].BandFreq[3] = 7000000L+3000L;
	VfoArray[1].BandFreq[4] = 10100000L;
	VfoArray[1].BandFreq[5] = 14000000L;
	VfoArray[1].BandFreq[6] = 18068000L;
	VfoArray[1].BandFreq[7] = 21000000L;
	VfoArray[1].BandFreq[8] = 24890000L;
	VfoArray[1].BandFreq[9] = 28000000L;
	VfoArray[1].BandFreq[10] = 50000000L;
	VfoArray[1].BandFreq[11] = 76100000L;
	VfoArray[1].Mode = MODE_ITEM_USB;	// USB
	VfoArray[1].IFBandWidth = 3 + FILTER_ITEM_NOT_WFM;
	VfoArray[1].AGC = 1;
	VfoArray[1].BandNo = 9;
	VfoArray[1].Flag.BIT.RfAmp = 1;
	// CurrentVfo	// +0
	//
	CurrentVfo = VfoArray[0];
	// RitOffset	// +b4
	RitOffset = 0L;
	// IFFreq	// +b8
	IFFreq = 10700000L;
	// CWPitch	// +bc
	CWPitch = 600;
	// ShiftFreq	// +be
	ShiftFreq = 0;
	
	// VfoNo	// +c0
	VfoNo = 0;
	// RFGain	// +c1
	RFGain = 50;
	// MicGain	// +c2
	MicGain = 3;
	// LevelSQ_Hysteresis	// +c3
	LevelSQ_Hysteresis = 0;
	// CurrentVoiceSQ_Kando	// +c4
	CurrentVoiceSQ_Kando = 0;
	// CurrentVoiceSQ_Delay	// +c5
	CurrentVoiceSQ_Delay = 0;
	// CurrentVoiceSQ_Enable	// +c6
	CurrentVoiceSQ_Enable = 0;
	// DeNoise	// +c7
	DeNoise = 0;
	// AutoNotch	// +c8
	AutoNotch = 0;
	// NoiseBlanker	// +c9
	NoiseBlanker = 0;
	// AfcVal	// +ca
	AfcVal = 0;
	// SemiBreakInHoldTime	// +cb
	SemiBreakInHoldTime = 10;
	// AMModulationDepth	// +cc
	AMModulationDepth = 50;
	// TXPower	// +cd
	TXPower = 10;
	// TXMeterType	// +ce
	TXMeterType = 0;
	// SuperHeteroFlag	// +cf
	SuperHeteroFlag = 0;
	// TestModeFlag		// +d0
	TestModeFlag = 0;
	// TestToneFlag		// +d1
	TestToneFlag = 0;
	// IFShiftEnable	// +d2
	IFShiftEnable = 0;
	// CWCarrier	// +d3
	CWCarrier = 50;
	// CWSideTone	// +d4
	CWSideTone = 20;
	// BeepVol	// +d5
	BeepVol = 4;
	// VolVal	// +d6
	VolVal = 10;
	// SQVal	// +d7
	SQVal = 0;

	
	backup_MemToBuf(bufptr);
}

//==========================================
//  backup_BufAddCRC
//      FlashBuf CRCǉ
//==========================================
void backup_BufAddCRC(unsigned char *bufptr)
{
unsigned char *p;
unsigned short i;
unsigned char v1,v2;
	flash_crc = get_crc_initial();
	p = bufptr;
	for(i=0; i<BACKUP_BUF_DATA_SIZE; i++){
		flash_crc = calc_crc16_1byte(flash_crc, *p++);
	}
	flash_crc = calc_last_crc(flash_crc);
	v1 = (unsigned char)(flash_crc & 0x00ff);
	v2 = (unsigned char)((flash_crc & 0xff00)>>8);
	*p++ = v1;
	*p++ = v2;
}

//==========================================
//  backup_BufCheckCRC
//      FlashBuf CRCmF
//		out  =True: OK  =False: NG
//==========================================
unsigned char backup_BufCheckCRC(unsigned char *bufptr)
{
unsigned char *p;
unsigned short i;
	flash_crc = get_crc_initial();
	p = bufptr;
	for(i=0; i<(BACKUP_BUF_DATA_SIZE + BACKUP_BUF_CRC_SIZE); i++){
		flash_crc = calc_crc16_1byte(flash_crc, *p++);
	}
	return check_crc(flash_crc);
}

//==========================================
//  backup_BufCheckValid
//      FlashBuf  valid check
//==========================================
unsigned char backup_BufCheckValid(unsigned char *bufptr)
{
unsigned char *p;
struct VfoParamStruct *vfop;
unsigned long *lp;
unsigned short *sp;
unsigned long lv;
long lsv;
unsigned short sv;
short ssv;
unsigned char result,r;
extern unsigned char vfo_check_valid(struct VfoParamStruct *vfop);
	result = (0==1);
	p = bufptr;
	
	// CurrentVfo	// +0
	vfop = (struct VfoParamStruct *)p;
	r = vfo_check_valid(vfop);
	if(r){
		vfop++;
	}else{
		return result;
	}
	// VfoA		// +3c
	r = vfo_check_valid(vfop);
	if(r){
		vfop++;
	}else{
		return result;
	}
	// VfoB		// +78
	r = vfo_check_valid(vfop);
	if(r){
		vfop++;
	}else{
		return result;
	}
	lp = (unsigned long *)vfop;
	// RitOffset	// +b4
	lv = *lp++;
	lsv = (long)lv;
	if((lsv < RIT_LOW_LIMIT)||(lsv > RIT_HIGH_LIMIT)){
		return result;
	}
	// IFFreq	// +b8
	lv = *lp++;
	if((lv < IF_FREQ_LOW_LIMIT)||(lv > IF_FREQ_HIGH_LIMIT)){
		return result;
	}
	sp = (unsigned short *)lp;
	// CWPitch	// +bc
	sv = *sp++;
	if((sv < CW_PITCH_MIN)||(sv > CW_PITCH_MAX)){
		return result;
	}
	// ShiftFreq	// +be
	sv = *sp++;
	ssv = (short)sv;
	if((ssv < SHIFT_FREQ_MIN)||(ssv > SHIFT_FREQ_MAX)){
		return result;
	}
	p = (unsigned char *)sp;
	
	// VfoNo	// +c0
	r = *p++;
	if(r > 1){
		return result;
	}
	// RFGain	// +c1
	r = *p++;
	if(r > RFGAIN_MAX){
		return result;
	}
	// MicGain	// +c2
	r = *p++;
	if(r > MIC_GAIN_MAX){
		return result;
	}
	// LevelSQ_Hysteresis	// +c3
	r = *p++;
	if(r > SQ_HYSTERESIS_MAX){
		return result;
	}
	// CurrentVoiceSQ_Kando	// +c4
	r = *p++;
	if(r > VSQ_KANDO_MAX){
		return result;
	}
	// CurrentVoiceSQ_Delay	// +c5
	r = *p++;
	if(r > VSQ_DELAY_MAX){
		return result;
	}
	// CurrentVoiceSQ_Enable	// +c6
	r = *p++;
	if(r > 1){
		return result;
	}
	// DeNoise	// +c7
	r = *p++;
	if(r > DENOISE_MAX){
		return result;
	}
	// AutoNotch	// +c8
	r = *p++;
	if(r > AUTO_NOTCH_MAX){
		return result;
	}
	// NoiseBlanker	// +c9
	r = *p++;
	if(r > 1){
		return result;
	}
	// AfcVal	// +ca
	r = *p++;
	if(r > 1){
		return result;
	}
	// SemiBreakInHoldTime	// +cb
	r = *p++;
	if(r > SEMI_BREAK_IN_MAX){
		return result;
	}
	// AMModulationDepth	// +cc
	r = *p++;
	if(r > AM_MODULATION_MAX){
		return result;
	}
	// TXPower	// +cd
	r = *p++;
	if(r > TX_POWER_MAX){
		return result;
	}
	// TXMeterType	// +ce
	r = *p++;
	if(r > TX_METER_TYPE_MAX){
		return result;
	}
	// SuperHeteroFlag	// +cf
	r = *p++;
	if(r > 1){
		return result;
	}
	// TestModeFlag		// +d0
	r = *p++;
	if(r > 1){
		return result;
	}
	// TestToneFlag		// +d1
	r = *p++;
	if(r > 1){
		return result;
	}
	// IFShiftEnable	// +d2
	r = *p++;
	if(r > 1){
		return result;
	}
	// CWCarrier	// +d3
	r = *p++;
	if(r > AM_MODULATION_MAX){
		return result;
	}
	// CWSideTone	// +d4
	r = *p++;
	if(r > VOL_MAX){
		return result;
	}
	// BeepVol	// +d5
	r = *p++;
	if(r > BEEP_VOL_MAX){
		return result;
	}
	
	
	result = (0==0);
	return result;
}

//==========================================
//  backup_timer
//      1ms^C}[荞ݏ
//==========================================
void backup_timer(void)
{
	if(backup_timer_flag==1){	// when now count
		if(backup_timer_count!=0){
			backup_timer_count--;
			if(backup_timer_count==0){
				backup_timer_flag = 0xff;
			}
		}
	}
}

//==========================================
//  backup_timer_start
//      backup timer start
//		in: wait count (1ms)
//==========================================
void backup_timer_start(unsigned short count)
{
	backup_timer_count = count;
	backup_timer_flag = 1;
}

//==========================================
//  backup_timer_stop
//      backup timer stop
//==========================================
void backup_timer_stop(void)
{
	backup_timer_count = 0;
	backup_timer_flag = 0;
}



#if defined(FLASH_API_RX_CFG_ENABLE_ROM_PROGRAMMING)
#pragma section FRAM
#endif

////==========================================
////  flash_test01
////		FlashñeXg 
////==========================================
//void flash_test01(void)
//{
//unsigned char *ptr;
//unsigned char r,i;
//
//
////backup_MemToBuf(backup_buf);
////backup_BufAddCRC(backup_buf);
////r=backup_BufCheckCRC(backup_buf);
////if(r){
//// r = backup_BufCheckValid(backup_buf);
//// if(r){
////  nop();
////  nop();
//// }else{
////  nop();
////  nop();
//// }
////}
////return;
//
////	memset(flash_read_buf, 0x00, 128);
////	memset(flash_write_buf, 0x00, 128);
//	for(i=0;i<128;i++){
//		flash_read_buf[i] = flash_write_buf[i] = 0x00;
//	}
//	
//retry:
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//	flash_is_ready = false;
//#endif
//	i = 0;
//	r = R_FlashDataAreaBlankCheck(DF_ADDRESS + 0x100, BLANK_CHECK_2_BYTE);
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//	while( flash_is_ready == false){
//		nop();
//		i++;
//		if(i==200){
//			nop();
//			blank_check_result = FLASH.DFLBCSTAT.BIT.BCST;	// koba
//			break;
//		}
//	}
//	r = blank_check_result;
//#endif
//	switch(r){
//		case FLASH_BLANK:
//			nop();
//			break;
//		case FLASH_NOT_BLANK:
//			nop();
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//			flash_is_ready = false;
//#endif
//			i=0;
//			r = R_FlashEraseRange(DF_ADDRESS + 0x100, 128);
//			nop();
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//			while( flash_is_ready == false){
//				nop();
//				i++;
//				if(i==200){
//					nop();
//					break;
//				}
//			}
//#endif
//			goto retry;
//			break;
//		case FLASH_FAILURE:
//			nop();
//			break;
//		case FLASH_BUSY:
//			nop();
//			break;
//		case FLASH_ERROR_ADDRESS:
//			nop();
//			break;
//		case FLASH_ERROR_BYTES:
//			nop();
//			break;
//		default:
//			nop();
//			break;
//	}
//	
////	strcpy(flash_write_buf,"0123ABCDabcd");
//	flash_write_buf[0] = '0';
//	flash_write_buf[1] = '1';
//	flash_write_buf[2] = '2';
//	flash_write_buf[3] = '3';
//	flash_write_buf[4] = 'A';
//	flash_write_buf[5] = 'B';
//	flash_write_buf[6] = 'C';
//	flash_write_buf[7] = 'D';
//	flash_write_buf[8] = 'a';
//	flash_write_buf[9] = 'b';
//	flash_write_buf[10] = 'c';
//	flash_write_buf[11] = 'd';
//	flash_write_buf[12] = '0';
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//	flash_is_ready = false;
//#endif
//	r = R_FlashWrite(DF_ADDRESS + 0x100, flash_write_buf, 8);
//	i = 0;
//	if(r == FLASH_SUCCESS){
//#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
//		while( flash_is_ready == false){
//			nop();
//			i++;
//			if(i==200){
//				nop();
//				break;
//			}
//		}
//#endif
//		ptr = DF_ADDRESS + 0x100;
//		for(i=0; i<32;i++){
//			flash_read_buf[i] = *ptr++;
//		}
//	}
//	nop();
//	
////	// ROM blank check
////	r = FLASH_BLANK;
////	ptr = 0xfffe0000;
////	for(i=0;i<16;i++){
////		if(*ptr != 0xff){
////			r = FLASH_NOT_BLANK;
////		}
////		flash_read_buf[i] = *ptr;
////		ptr++;
////	}
////	nop();
/////	
////#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
////	flash_is_ready = false;
////#endif
////	i = 0;
////	r = R_FlashWrite(0xfffe0000,flash_write_buf,8);
////	nop();
////	
////	if(r == FLASH_SUCCESS){
////#if defined(FLASH_API_RX_CFG_DATA_FLASH_BGO)
////		while( flash_is_ready == false){
////			nop();
////			i++;
////		}
////#endif
////		ptr = 0xfffe0000;
////		for(i=0; i<32;i++){
////			flash_read_buf[i] = *ptr++;
////		}
////	}
//	nop();
//}



