/***********************************************************************/
/*                                                                     */
/*  FILE        :lcd.c                                                 */
/*  DESCRIPTION :LCD High Level Procedure                              */
/*  CPU TYPE    :RX210                                                 */
/*                                                                     */
/*  last modefied : Dec 19, 2013                                       */
/*  for CQ-HF Panel   AOR  by KOBAYASHI Ikuo                           */
/*                                                                     */
/***********************************************************************/
                  
//#include "variation.h"
#include <machine.h>
#include <string.h>
#include <stdio.h>
#include "iodefine.h"
#include "display.h"
#include "vfo_param.h"

#ifdef __cplusplus
extern "C" {
#endif

#define	PANEL_VER_STR	"412E"

#define	LCD_POPUP_TIME	(3000)

void lcd_main(void);
void lcd_timer(void);

void lcd_disp_TX_icon(unsigned char val);
void lcd_disp_Swr(unsigned char val);
void lcd_disp_Busy_icon(unsigned char val);
void lcd_disp_RfAmp_icon(unsigned char val);
void lcd_disp_IFShift_icon(unsigned char enable, short freq);
void lcd_disp_Agc_icon(unsigned char val);
void lcd_disp_Filter_icon(unsigned char val);
void lcd_disp_Rit_icon(long freq);
void lcd_disp_Mode_icon(unsigned char val);
void lcd_disp_VfoNo_icon(unsigned char val);
void lcd_disp_RxFreq(unsigned long freq);
void lcd_disp_RitOffset(long freq);
void lcd_disp_IFShift(short val);
void lcd_disp_FreqCursor(unsigned char val);
void lcd_disp_set_Cursor(void);
void lcd_disp_AllRefresh(void);

void lcd_disp_popup_start(void);
void lcd_disp_popup_stop(unsigned char val);
void lcd_disp_Cursor_popup(void);
void lcd_disp_RfAmp_popup(unsigned char val);
void lcd_disp_Agc_popup(unsigned char val);
void lcd_disp_Filter_popup(unsigned char val);
void lcd_disp_Mode_popup(unsigned char val);
void lcd_disp_AG_popup(unsigned char val);
void lcd_disp_SQ_popup(unsigned char val);
void lcd_disp_RFGain_popup(unsigned char val);
void lcd_disp_IFShift_popup(short val);
void lcd_disp_Dial_popup(unsigned long val,unsigned char mode);
void lcd_disp_Rit_popup(long val);

void lcd_disp_MenuTitle(unsigned char val);
void lcd_disp_MenuParam(unsigned char val);
void lcd_disp_SetCursor_MenuNo(void);
void lcd_disp_SetCursor_MenuParam(void);
void lcd_disp_SetCursor_Menu(void);
static void lcd_disp_MenuParam_SQ_Hys(void);
static void lcd_disp_MenuParam_VSQ(void);
static void lcd_disp_MenuParam_VSQ_Kando(void);
static void lcd_disp_MenuParam_VSQ_Delay(void);
static void lcd_disp_MenuParam_DENOISE(void);
static void lcd_disp_MenuParam_AutoNOTCH(void);
static void lcd_disp_MenuParam_NoiseBlanker(void);
static void lcd_disp_MenuParam_CWPitch(void);
static void lcd_disp_MenuParam_CWSideToneLevel(void);
static void lcd_disp_MenuParam_CWCarrierLevel(void);
static void lcd_disp_MenuParam_SemiBreakIn_HoldTime(void);
static void lcd_disp_MenuParam_AFC(void);
static void lcd_disp_MenuParam_IF_Shift(void);
static void lcd_disp_MenuParam_MicGain(void);
static void lcd_disp_MenuParam_AMModulation(void);
static void lcd_disp_MenuParam_TXPower(void);
static void lcd_disp_MenuParam_TXMeter(void);
static void lcd_disp_MenuParam_SuperHetero(void);
static void lcd_disp_MenuParam_IF_Freq(void);
static void lcd_disp_MenuParam_TestMode(void);
static void lcd_disp_MenuParam_TestTone(void);
static void lcd_disp_MenuParam_BeepVol(void);
static void lcd_disp_MenuParam_Reset(void);
void lcd_disp_MenuParam_Reset_Complete(void);
static void lcd_disp_MenuParam_Version(void);





#ifdef __cplusplus
}
#endif

unsigned short lcd_timer_counter;

unsigned char lcd_timer_flag;
unsigned char NowPopUp,NowCursorPopUp;
char tmpbuf[16];

extern void lcd_cls(void);
extern void lcd_locate(unsigned char x, unsigned char y, unsigned char cursor);
extern void lcd_putchar(char c);
extern void lcd_print(char *ptr);
extern unsigned char vfo_Freq_to_Band(unsigned long freq, unsigned char *band);
extern unsigned char vfo_Freq_to_NextBand(unsigned long freq);
extern unsigned char vfo_isAmatureFreq(unsigned long freq,unsigned char mode);


extern struct VfoParamStruct CurrentVfo,VfoParamA,VfoParamB;

extern long RitOffset;
extern short ShiftFreq;

extern unsigned char MainMode;
extern unsigned char VfoNo;
extern unsigned char RitInputFlag;
extern unsigned char SQOpened;
extern unsigned char RFGain;
extern unsigned char FreqCursorPos,RitCursorPos;
extern unsigned char IfFreqInputFlag,IfFreqCursorPos;
extern unsigned char AMModulationDepth,TXPower;
extern unsigned char TXMeterType;
extern unsigned char TestModeFlag,TestToneFlag;
extern unsigned char IFShiftEnable;
extern unsigned char CWCarrier,CWSideTone;



char *LcdTxIconStr[]={"  ","TX","--"};
char *LcdAgcIconStr[]={" ","S","M","F"};
char *LcdAgcStr[]={"OFF ","SLOW","MID ","FAST"};
char *LcdFilterStr[]={"300k","200k","100k","30k ","15k "," 6k "," 3k ","300 "};
char *LcdModeStr[]={"WFM","FM ","AM ","SAM","LSB","USB","CW "};

char *LcdOffLowMidHighStr[]={" OFF"," LOW"," MID","HIGH"};
char *LcdOffOnStr[]={"OFF"," ON"};

char *LcdMenuTitle[] = {
	"   SQ Hysteresis",	// 01:SQ Hys.
	"   Voice Squelch",	// 02:VSQ
	" VSQ Sensitivity",	// 03:VSQ Kando
	"       VSQ Delay",	// 04:VSQ Delay
	"          DENOIS",	// 05:Denoise
	"      Auto NOTCH",	// 06:AutoNotch
	"   Noise Blanker",	// 07:NoiseBlanker
	"        CW Pitch",	// 08:CWPitch
	"CW SideToneLevel",	// 09:CW SideToneLevel
	"CW Carrier Level",	// 10:CW CarrierLevel
	"   CW Delay Time",	// 11:CW DelayTime (old:SemiBreakIn HoldTime)
	"      AFC at NFM",	// 12:AFC
	"        IF Shift",	// 13:IF Shift
	"        MIC Gain",	// 14:MIC Gain
	"   AM Modulation",	// 15:AM Modulation Depth
	"        TX Power",	// 16:TX Power
	"        TX Meter",	// 17:TX Meter
	"Super-Heterodyne",	// 18:SuperHetero
	" Hetero. IF Freq",	// 19:IF Freq
	"       Test Mode",	// 20:TestMode
	"       Test Tone",	// 21:TestTone
	"        Beep Vol",	// 22:BeepVol
	"           Reset",	// 23:Reset
	"         Version",	// 24:Version

};

MENU_FUNC	MenuDispParam[] = {
	lcd_disp_MenuParam_SQ_Hys,		// 01:SQ Hys.
	lcd_disp_MenuParam_VSQ,			// 02:VSQ
	lcd_disp_MenuParam_VSQ_Kando,	// 03:VSQ Kando
	lcd_disp_MenuParam_VSQ_Delay,	// 04:VSQ Delay
	lcd_disp_MenuParam_DENOISE,		// 05:Denoise
	lcd_disp_MenuParam_AutoNOTCH,	// 06:AutoNotch
	lcd_disp_MenuParam_NoiseBlanker,	// 07:NoiseBlanker
	lcd_disp_MenuParam_CWPitch,		// 08:CWPitch
	lcd_disp_MenuParam_CWSideToneLevel,	// 09:CWSideToneLevel
	lcd_disp_MenuParam_CWCarrierLevel,	// 10:CWCarrierLevel
	lcd_disp_MenuParam_SemiBreakIn_HoldTime,	// 11:SemiBreakIn HoldTime
	lcd_disp_MenuParam_AFC,			// 12:AFC
	lcd_disp_MenuParam_IF_Shift,	// 13:IF Shift
	lcd_disp_MenuParam_MicGain,		// 14: MIC Gain
	lcd_disp_MenuParam_AMModulation,	// 15: AM Modulation
	lcd_disp_MenuParam_TXPower,		// 16: TX Power
	lcd_disp_MenuParam_TXMeter,		// 17: TX Meter
	lcd_disp_MenuParam_SuperHetero,	// 18: SuperHeterodyne
	lcd_disp_MenuParam_IF_Freq,		// 19: IF Freq
	lcd_disp_MenuParam_TestMode,	// 20: TestMode
	lcd_disp_MenuParam_TestTone,	// 21: TestTone
	lcd_disp_MenuParam_BeepVol,		// 22: BeepVol
	lcd_disp_MenuParam_Reset,		// 23: Reset
	lcd_disp_MenuParam_Version,		// 24: Version

};

//==========================================
//  lcd_main
//		LCD   display main
//==========================================
void lcd_main(void)
{
	if( (lcd_timer_flag & 0x80)!= 0){
		lcd_disp_popup_stop(1);
	}
}

//==========================================
//  lcd_timer
//		LCD   1ms interrupt
//==========================================
void lcd_timer(void)
{
	if( (lcd_timer_flag & 0x01)!= 0){
		if(lcd_timer_counter != 0){
			lcd_timer_counter --;
			if(lcd_timer_counter == 0){
				lcd_timer_flag &= ~0x01;
				lcd_timer_flag |= 0x80;
			}
		}
	}
}

//==========================================
//  lcd_disp_popup_start
//		LCD   popup timer start
//==========================================
void lcd_disp_popup_start(void)
{
	lcd_timer_counter = LCD_POPUP_TIME;
	lcd_timer_flag |= 0x01;
	NowPopUp = 1;
}

//==========================================
//  lcd_disp_popup_stop
//		LCD   popup timer stop
//==========================================
void lcd_disp_popup_stop(unsigned char flag)
{
unsigned long freq;
extern unsigned char MenuNo;
extern unsigned char NowTxFlag;
	lcd_timer_counter = 0;
	lcd_timer_flag &= ~0x81;
	NowPopUp = 0;
	NowCursorPopUp = 0;
	if(flag==1){
		switch(MainMode){
			case MAIN_MODE_VFO:
				if(RitInputFlag==0){
					freq = CurrentVfo.CurrentFreq;
					if(NowTxFlag == 0){
						freq += RitOffset;
					}
					lcd_disp_RxFreq(freq/*CurrentVfo.CurrentFreq*/);
//					lcd_disp_FreqCursor(FreqCursorPos);
				}else{
					lcd_disp_RitOffset(RitOffset);
//					lcd_disp_FreqCursor(RitCursorPos);
				}
				break;
			case MAIN_MODE_MENU_SEL:
				lcd_disp_MenuTitle(MenuNo);
				lcd_disp_MenuParam(MenuNo);
				lcd_disp_SetCursor_MenuNo();
				break;
			case MAIN_MODE_MENU_EDIT:
				lcd_disp_MenuTitle(MenuNo);
				lcd_disp_MenuParam(MenuNo);
				lcd_disp_SetCursor_MenuParam();
				break;
		}
	}
}


//==========================================
//  lcd_disp_TX_icon
//		LCD   TX disp
//		in	val: =0:space  =1:TX  =0xff:--
//==========================================
void lcd_disp_TX_icon(unsigned char val)
{
	lcd_locate(0,0,0);
	if(val==0xff){
		val = 2;
	}
	lcd_print(LcdTxIconStr[val]);
}

//==========================================
//  lcd_disp_Swr
//		LCD   SWR disp
//		in	val: 10-99=SWR(x0.1)  =0xff:space
//==========================================
void lcd_disp_Swr(unsigned char val)
{
unsigned char v1,v2;
	lcd_locate(2,0,0);
	if((val>=10)&&(val<=99)){
		v1 = val / 10;
		v2 = val % 10;
		sprintf(tmpbuf,"%d.%d",v1,v2);
	}else{
		sprintf(tmpbuf,"   ");
	}
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_Busy_icon
//		LCD   Busy disp
//		in	val: =0:clear  =1:BUSY
//==========================================
void lcd_disp_Busy_icon(unsigned char val)
{
char c;
	if(val == 1){
		c = 'B';
	}else{
		c = ' ';
	}
	lcd_locate(5,0,0);
	lcd_putchar(c);
}

//==========================================
//  lcd_disp_RfAmp_icon
//		LCD   RfAmp disp
//		in	val: =0:clear  =1:AMP
//==========================================
void lcd_disp_RfAmp_icon(unsigned char val)
{
char c;
	if(val == 1){
		c = 'A';
	}else{
		c = ' ';
	}
	lcd_locate(7,0,0);
	lcd_putchar(c);
}

//==========================================
//  lcd_disp_IFShift_icon
//		LCD   IFShift disp
//		in	enable: =0:clear  =1:Shift
//==========================================
void lcd_disp_IFShift_icon(unsigned char enable, short freq)
{
char c;
	if(enable == 0){
		c = ' ';
	}else{
		if(freq==0){
			c = 0xa5;
		}else if(freq < 0){
			c = 0x7f;
		}else{
			c = 0x7e;
		}
	}
	lcd_locate(8,0,0);
	lcd_putchar(c);
}

//==========================================
//  lcd_disp_Agc_icon
//		LCD   AGC disp
//		in	val: =0:space(OFF)  =1:S(SLOW)  =2:F(FAST)
//==========================================
void lcd_disp_Agc_icon(unsigned char val)
{
	lcd_locate(9,0,0);
	lcd_print(LcdAgcIconStr[val]);
}

//==========================================
//  lcd_disp_Filter_icon
//		LCD   IF Filter disp
//		in	val:
//char *LcdFilterStr[]={"300k","200k","100k","30k ","15k "," 6k "," 3k ","300 "};
//==========================================
void lcd_disp_Filter_icon(unsigned char val)
{
	lcd_locate(11,0,0);
	lcd_print(LcdFilterStr[val]);
}

//==========================================
//  lcd_disp_Rit_icon
//		LCD   RIT disp
//		in	val: =0:clear  =1:RITY
//==========================================
void lcd_disp_Rit_icon(long freq)
{
char c;
	if(freq == 0L){
		c = ' ';
	}else{
		c = 'R';
	}
	lcd_locate(15,0,0);
	lcd_putchar(c);
}

//==========================================
//  lcd_disp_Mode_icon
//		LCD   Mode disp
//		in	val:  =0:WFM  =1:FM  =2:EAM  =3:SAM  =4:LSB  =5:USB  =6:CW 
//==========================================
void lcd_disp_Mode_icon(unsigned char val)
{
	lcd_locate(0,1,0);
	lcd_print(LcdModeStr[val]);
}

//==========================================
//  lcd_disp_VfoNo_icon
//		LCD   VfoNo disp
//		in	val: =0:A  =1:B
//==========================================
void lcd_disp_VfoNo_icon(unsigned char val)
{
char c;
	if(val == 0){
		c = 'A';
	}else{
		c = 'B';
	}
	lcd_locate(4,1,0);
	lcd_putchar(c);
}

//==========================================
//  lcd_disp_RxFreq
//		LCD   RxFreq disp
//		in	freq:
//==========================================
void lcd_disp_RxFreq(unsigned long freq)
{
unsigned long f1,f2,f3;
	f1 = freq / 1000000L;
	f3 = freq % 1000000L;
	f2 = f3 / 1000L;
	f3 = f3 % 1000L;
	lcd_locate(6,1,0);
	sprintf(tmpbuf,"%2d.%03d,%03d",f1,f2,f3);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_RitOffset
//		LCD   RxFreq disp
//		in	freq:
//==========================================
void lcd_disp_RitOffset(long freq)
{
unsigned long f1,f2;
char c;
	if(freq == 0){
		c = ' ';
	}else if(freq > 0){
		c = '+';
	}else{
		c = '-';
		freq = -freq;
	}
	f1 = freq / 1000L;
	f2 = freq % 1000L;
	lcd_locate(6,1,0);
	sprintf(tmpbuf,"RIT:%c%01d,%03d",c,f1,f2);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_IFShift
//		LCD   IFShift disp
//		in	val:
//==========================================
void lcd_disp_IFShift(short val)
{
unsigned short f1,f2;
char c;
	if(val == 0){
		c = ' ';
	}else if(val > 0){
		c = '+';
	}else{
		c = '-';
		val = -val;
	}
	f1 = val / 1000L;
	f2 = val % 1000L;
	lcd_locate(6,1,0);
	sprintf(tmpbuf,"IFS:%c%01d,%03d",c,f1,f2);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_FreqCursor
//		LCD   Freq cursor  disp
//		in	pos:
//==========================================
void lcd_disp_FreqCursor(unsigned char pos)
{
unsigned char CursorPos[] = {15,14,13,11,10,9,7,6};
	lcd_locate(CursorPos[pos],1,NowCursorPopUp/*1*/);
}

//==========================================
//  lcd_disp_set_Cursor
//		LCD   Freq cursor  disp
//		in	pos:
//==========================================
void lcd_disp_set_Cursor(void)
{
	if(MainMode == MAIN_MODE_VFO){
//		if(NowPopUp == 0){
			if(RitInputFlag==0){
//				lcd_disp_RxFreq(CurrentVfo.CurrentFreq);
				lcd_disp_FreqCursor(FreqCursorPos);
			}else{
//				lcd_disp_RitOffset(RitOffset);
				lcd_disp_FreqCursor(RitCursorPos);
			}
//		}
	}
}

//==========================================
//  lcd_disp_AllRefresh
//		LCD   Rxʁ@SXV
//==========================================
void lcd_disp_AllRefresh(void)
{
unsigned long freq;
unsigned char t;
extern unsigned char NowTxFlag;
extern unsigned char swr_buf;
	lcd_cls();
	if(MainMode == MAIN_MODE_INIT){
		lcd_locate(0,0,0);
//		lcd_print("                ");
		lcd_print(" TRX-305        ");
		lcd_locate(0,1,0);
		lcd_print("   just a moment");
		return;
	}
	
	if(NowTxFlag==0){
		t = vfo_isAmatureFreq(CurrentVfo.CurrentFreq,CurrentVfo.Mode) ? 0: 2;
	}else{
		t = 1;
	}
	lcd_disp_TX_icon(t);
	if((NowTxFlag != 0)/*&&(TXMeterType==2)*/){
		t = swr_buf;
	}else{
		t = 0xff;
	}
	lcd_disp_Swr(t);
	lcd_disp_Busy_icon(SQOpened);
	lcd_disp_RfAmp_icon(CurrentVfo.Flag.BIT.RfAmp);
	lcd_disp_IFShift_icon(IFShiftEnable,ShiftFreq);
	lcd_disp_Agc_icon(CurrentVfo.AGC);
	t = CurrentVfo.IFBandWidth;
	if(CurrentVfo.Mode==MODE_ITEM_CW){	// when CW
		t = FILTER_ITEM_MAX;
	}
	lcd_disp_Filter_icon(t);
	lcd_disp_Rit_icon(RitOffset);
	lcd_disp_Mode_icon(CurrentVfo.Mode);
	lcd_disp_VfoNo_icon(VfoNo);
	if(RitInputFlag==0){
		freq = CurrentVfo.CurrentFreq;
		if(NowTxFlag == 0){
			freq += RitOffset;
		}
		lcd_disp_RxFreq(freq/*CurrentVfo.CurrentFreq*/);
//		lcd_disp_FreqCursor(FreqCursorPos);
	}else{
		lcd_disp_RitOffset(RitOffset);
//		lcd_disp_FreqCursor(RitCursorPos);
	}
}

//==========================================
//  lcd_disp_Cursor_popup
//		LCD   Cursor changing disp
//==========================================
void lcd_disp_Cursor_popup(void)
{
	NowCursorPopUp = 1;
//	lcd_disp_popup_start();
	lcd_timer_counter = LCD_POPUP_TIME;
	lcd_timer_flag |= 0x01;
}

//==========================================
//  lcd_disp_RfAmp_popup
//		LCD   RfAmp changing disp
//		in	val: =0:off  =1:on
//==========================================
void lcd_disp_RfAmp_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print("RFAmp: ");
	lcd_print(LcdOffOnStr[val]);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_Agc_popup
//		LCD   AGC chenging disp
//		in	val: =0:off  =1:slow  =2:fast
//==========================================
void lcd_disp_Agc_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print(" AGC: ");
	lcd_print(LcdAgcStr[val]);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_Filter_popup
//		LCD   IF Filter changing disp
//		in	val:
//==========================================
void lcd_disp_Filter_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	if(val >= FILTER_ITEM_NOT_WFM){
		lcd_print("Filter:");
	}else{
		lcd_print("Filt.:");
	}
	lcd_print(LcdFilterStr[val]);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_Mode_popup
//		LCD   Mode changing disp
//		in	val: =0:FM  =1:AM  =2:USB  =3:LSB  =4:CW
//==========================================
void lcd_disp_Mode_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print(" Mode: ");
	lcd_print(LcdModeStr[val]);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_AG_popup
//		LCD   AG changing disp
//		in	val:
//==========================================
void lcd_disp_AG_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print("  AG : ");
	sprintf(tmpbuf,"%3d",val);
	lcd_print(tmpbuf);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_SQ_popup
//		LCD   SQ changing disp
//		in	val:
//==========================================
void lcd_disp_SQ_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print("  SQ : ");
	sprintf(tmpbuf,"%3d",val);
	lcd_print(tmpbuf);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_RFGain_popup
//		LCD   RFGain changing disp
//		in	val:
//==========================================
void lcd_disp_RFGain_popup(unsigned char val)
{
	lcd_locate(6,1,0);
	lcd_print("  RF : ");
	sprintf(tmpbuf,"%3d",val);
	lcd_print(tmpbuf);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_IFShift_popup
//		LCD   IFShift changing disp
//		in	val:
//==========================================
void lcd_disp_IFShift_popup(short val)
{
	lcd_disp_IFShift_icon(IFShiftEnable,ShiftFreq);
	lcd_disp_IFShift(ShiftFreq);
	NowCursorPopUp = 0;
	lcd_disp_popup_start();
}

//==========================================
//  lcd_disp_Dial_popup
//		LCD   Dial changing disp
//		in	val:
//			mode
//==========================================
void lcd_disp_Dial_popup(unsigned long val, unsigned char mode)
{
unsigned char v;
extern unsigned char NowTxFlag,TXMeterType;
extern unsigned char swr_buf;
	lcd_disp_popup_stop(0);
	if(NowTxFlag == 0){
		v = vfo_isAmatureFreq(val,mode) ? 0: 2;
	}else{
		v = 1;
	}
	lcd_disp_TX_icon(v);
	if((NowTxFlag != 0)/*&&(TXMeterType==2)*/){
		v = swr_buf;
	}else{
		v = 0xff;
	}
	lcd_disp_Swr(v);
	if(NowTxFlag==0){
		val += RitOffset;
	}
	lcd_disp_RxFreq(val);
	NowCursorPopUp = 0;
	lcd_disp_FreqCursor(FreqCursorPos);
}

//==========================================
//  lcd_disp_Rit_popup
//		LCD   Rit changing disp
//		in	val:
//==========================================
void lcd_disp_Rit_popup(long val)
{
unsigned char v;
extern unsigned char NowTxFlag,TXMeterType;
	lcd_disp_popup_stop(0);
	if(NowTxFlag == 0){
		v = vfo_isAmatureFreq(CurrentVfo.CurrentFreq,CurrentVfo.Mode) ? 0: 2;
	}else{
		v = 1;
	}
	lcd_disp_TX_icon(v);
	
	lcd_disp_Rit_icon(RitOffset);
	lcd_disp_RitOffset(val);
	NowCursorPopUp = 0;
	lcd_disp_FreqCursor(RitCursorPos);
}

//==========================================
//  lcd_disp_MenuTitle
//		LCD   Menu title disp
//		in	val: MenuPage
//==========================================
void lcd_disp_MenuTitle(unsigned char val)
{
	lcd_locate(0,0,0);
	lcd_print(LcdMenuTitle[val]);
	lcd_locate(0,1,0);
	sprintf(tmpbuf,"M:%02d",val+1);
	lcd_print(tmpbuf);
	lcd_print("            ");
}

//==========================================
//  lcd_disp_MenuParam
//		LCD   Menu param disp
//		in	val: MenuPage
//==========================================
void lcd_disp_MenuParam(unsigned char val)
{
	MenuDispParam[val]();
}

//==========================================
//  lcd_disp_SetCursor_MenuNo
//		LCD   cursor set to MenuNo
//==========================================
void lcd_disp_SetCursor_MenuNo(void)
{
	lcd_locate(3,1,1);
}

//==========================================
//  lcd_disp_SetCursor_MenuParam
//		LCD   cursor set to MenuParam
//==========================================
void lcd_disp_SetCursor_MenuParam(void)
{
	if(IfFreqInputFlag==0){
		lcd_locate(15,1,1);
	}else{
		lcd_disp_FreqCursor(IfFreqCursorPos);
	}
}

//==========================================
//  lcd_disp_SetCursor_Menu
//		LCD   cursor set to Menu܂Ƃ
//==========================================
void lcd_disp_SetCursor_Menu(void)
{
	switch(MainMode){
		case MAIN_MODE_MENU_SEL:
			lcd_disp_SetCursor_MenuNo();
			break;
		case MAIN_MODE_MENU_EDIT:
			lcd_disp_SetCursor_MenuParam();
			break;
	}
}

//==========================================
//  lcd_disp_MenuParam_SQ_Hys
//		LCD   disp MenuParam SQ Hys.
//==========================================
static void lcd_disp_MenuParam_SQ_Hys(void)
{
extern unsigned char LevelSQ_Hysteresis;
	if(LevelSQ_Hysteresis==0){
		sprintf(tmpbuf,"OFF");
	}else{
		sprintf(tmpbuf,"%01ddB",LevelSQ_Hysteresis);
	}
	lcd_locate(13,1,0);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_VSQ
//		LCD   disp MenuParam VSQ
//==========================================
static void lcd_disp_MenuParam_VSQ(void)
{
extern unsigned char CurrentVoiceSQ_Enable;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[CurrentVoiceSQ_Enable]);
}

//==========================================
//  lcd_disp_MenuParam_VSQ_Kando
//		LCD   disp MenuParam VSQ Kando
//==========================================
static void lcd_disp_MenuParam_VSQ_Kando(void)
{
extern unsigned char CurrentVoiceSQ_Kando;
	sprintf(tmpbuf,"%01d",CurrentVoiceSQ_Kando);
	lcd_locate(15,1,0);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_VSQ_Delay
//		LCD   disp MenuParam VSQ Delay
//==========================================
static void lcd_disp_MenuParam_VSQ_Delay(void)
{
extern unsigned char CurrentVoiceSQ_Delay;
	sprintf(tmpbuf,"%2d",CurrentVoiceSQ_Delay);
	lcd_locate(14,1,0);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_DENOISE
//		LCD   disp MenuParam DENOISE
//==========================================
static void lcd_disp_MenuParam_DENOISE(void)
{
extern unsigned char DeNoise;
	lcd_locate(12,1,0);
	lcd_print(LcdOffLowMidHighStr[DeNoise]);
}

//==========================================
//  lcd_disp_MenuParam_AutoNOTCH
//		LCD   disp MenuParam AutoNOTCH
//==========================================
static void lcd_disp_MenuParam_AutoNOTCH(void)
{
extern unsigned char AutoNotch;
	lcd_locate(12,1,0);
	lcd_print(LcdOffLowMidHighStr[AutoNotch]);
}

//==========================================
//  lcd_disp_MenuParam_NoiseBlanker
//		LCD   disp MenuParam NoiseBlanker
//==========================================
static void lcd_disp_MenuParam_NoiseBlanker(void)
{
extern unsigned char NoiseBlanker;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[NoiseBlanker]);
}

//==========================================
//  lcd_disp_MenuParam_CWPitch
//		LCD   disp MenuParam CWPitch
//==========================================
static void lcd_disp_MenuParam_CWPitch(void)
{
extern unsigned short CWPitch;
	lcd_locate(10,1,0);
	sprintf(tmpbuf,"%4dHz",CWPitch);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_CWSideToneLevel
//		LCD   disp MenuParam CW SideToneLevel
//==========================================
static void lcd_disp_MenuParam_CWSideToneLevel(void)
{
extern unsigned char CWSideTone;
	lcd_locate(13,1,0);
	sprintf(tmpbuf,"%3d",CWSideTone);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_CWCarrierLevel
//		LCD   disp MenuParam CW CarrierLevel
//==========================================
static void lcd_disp_MenuParam_CWCarrierLevel(void)
{
extern unsigned char CWCarrier;
	lcd_locate(13,1,0);
	sprintf(tmpbuf,"%3d",CWCarrier);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_SemiBreakIn_HoldTime
//		LCD   disp MenuParam SemiBreakIn HoldTime
//==========================================
static void lcd_disp_MenuParam_SemiBreakIn_HoldTime(void)
{
unsigned char v1,v2;
extern unsigned char SemiBreakInHoldTime;
	v1 = SemiBreakInHoldTime / 10;
	v2 = SemiBreakInHoldTime % 10;
	lcd_locate(11,1,0);
	sprintf(tmpbuf,"%2d.%01dS",v1,v2);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_AFC
//		LCD   disp MenuParam AFC
//==========================================
static void lcd_disp_MenuParam_AFC(void)
{
extern unsigned char AfcVal;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[AfcVal]);
}

////==========================================
////  lcd_disp_MenuParam_IF_Shift
////		LCD   disp MenuParam IF Shift
////==========================================
//static void lcd_disp_MenuParam_IF_Shift(void)
//{
//extern short ShiftFreq;
//	lcd_locate(9,1,0);
//	sprintf(tmpbuf,"%5dHz",ShiftFreq);
//	lcd_print(tmpbuf);
//}

//==========================================
//  lcd_disp_MenuParam_IF_Shift
//		LCD   disp MenuParam IF Shift
//==========================================
static void lcd_disp_MenuParam_IF_Shift(void)
{
extern unsigned char IFShiftEnable;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[IFShiftEnable]);
}

//==========================================
//  lcd_disp_MenuParam_MicGain
//		LCD   disp MenuParam MIC Gain
//==========================================
static void lcd_disp_MenuParam_MicGain(void)
{
extern unsigned char MicGain;
	lcd_locate(15,1,0);
	sprintf(tmpbuf,"%d",MicGain);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_AMModulation
//		LCD   disp MenuParam AM Modulation Depth
//==========================================
static void lcd_disp_MenuParam_AMModulation(void)
{
extern unsigned char AMModulationDepth;
	lcd_locate(13,1,0);
	sprintf(tmpbuf,"%3d",AMModulationDepth);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_TXPower
//		LCD   disp MenuParam TX Power
//==========================================
static void lcd_disp_MenuParam_TXPower(void)
{
extern unsigned char TXPower;
	lcd_locate(13,1,0);
	sprintf(tmpbuf,"%3d",TXPower);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_TXMeter
//		LCD   disp MenuParam TX Power
//==========================================
static void lcd_disp_MenuParam_TXMeter(void)
{
//char * TxMeterStr[] = {"TX Power","     ALC","     SWR"};
char * TxMeterStr[] = {"TX Power","     ALC"};
extern unsigned char TXMeterType;
	lcd_locate(8,1,0);
	lcd_print(TxMeterStr[TXMeterType]);
}

//==========================================
//  lcd_disp_MenuParam_SuperHetero
//		LCD   disp MenuParam SuperHeterodyne
//==========================================
static void lcd_disp_MenuParam_SuperHetero(void)
{
extern unsigned char SuperHeteroFlag;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[SuperHeteroFlag]);
}

//==========================================
//  lcd_disp_MenuParam_IF_Freq
//		LCD   disp MenuParam IF_Freq
//==========================================
static void lcd_disp_MenuParam_IF_Freq(void)
{
extern unsigned long IFFreq;
	lcd_disp_RxFreq(IFFreq);
	lcd_disp_FreqCursor(IfFreqCursorPos);
}

//==========================================
//  lcd_disp_MenuParam_TestMode
//		LCD   disp MenuParam TestMode
//==========================================
static void lcd_disp_MenuParam_TestMode(void)
{
extern unsigned char TestModeFlag;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[TestModeFlag]);
}

//==========================================
//  lcd_disp_MenuParam_TestTone
//		LCD   disp MenuParam TestTone
//==========================================
static void lcd_disp_MenuParam_TestTone(void)
{
extern unsigned char TestToneFlag;
	lcd_locate(13,1,0);
	lcd_print(LcdOffOnStr[TestToneFlag]);
}

//==========================================
//  lcd_disp_MenuParam_BeepVol
//		LCD   disp MenuParam BeepVol
//==========================================
static void lcd_disp_MenuParam_BeepVol(void)
{
extern unsigned char BeepVol;
	sprintf(tmpbuf,"  %d",BeepVol);
	lcd_locate(13,1,0);
	lcd_print(tmpbuf);
}

//==========================================
//  lcd_disp_MenuParam_Reset
//		LCD   disp MenuParam Reset
//==========================================
static void lcd_disp_MenuParam_Reset(void)
{
	lcd_locate(8,1,0);
	lcd_print(" Execute");
}

//==========================================
//  lcd_disp_MenuParam_Reset_Complete
//		LCD   disp MenuParam ResetComplete
//==========================================
void lcd_disp_MenuParam_Reset_Complete(void)
{
	lcd_locate(8,1,0);
	lcd_print("Complete");
}

//==========================================
//  lcd_disp_MenuParam_Version
//		LCD   disp MenuParam Version
//==========================================
static void lcd_disp_MenuParam_Version(void)
{
extern char dem_version[16];

	lcd_locate(5,1,0);
	sprintf(tmpbuf,"M");
	strncat(tmpbuf,dem_version,4);
	lcd_print(tmpbuf);
	lcd_locate(10,1,0);
	lcd_print(" P");
	lcd_print(PANEL_VER_STR);
}



