/***********************************************************************/
/*                                                                     */
/*  FILE        :panel.c                                               */
/*  DESCRIPTION :Panel scan                                            */
/*  CPU TYPE    :RX210                                                 */
/*                                                                     */
/*  last modefied : Dec 17, 2013                                       */
/*  for CQ-HF Panel   AOR  by KOBAYASHI Ikuo                           */
/*                                                                     */
/***********************************************************************/
                  
#include <machine.h>
#include "iodefine.h"

//#include "display.h"
//#include "vfo_param.h"

#ifdef __cplusplus
extern "C" {
#endif

void panel_scan_initialize(void);
void panel_scan(void);
static void panel_scan_0(void);
static void panel_scan_1(void);
static void panel_scan_2(void);
static void panel_scan_3(void);
static void panel_scan_4(void);
static void panel_scan_5(void);

static void panel_off_enter(unsigned char no);
static void panel_on_enter(unsigned char no);
static void panel_long_push_enter(void);
void panel_on_continue(unsigned char no);

void panel_timer(void);
void panel_timer_start(unsigned short count);
void panel_timer_stop(void);
void sw_test(unsigned char v, char *ptr);

#ifdef __cplusplus
}
#endif

unsigned short panel_dial_Dial_old, panel_dial_AG_old, panel_dial_SQ_old;
unsigned short panel_old_map,panel_new_map,panel_tmp_map;
unsigned char panel_scan_state;
unsigned char panel_chat_count;
static unsigned char panel_timer_flag;
static unsigned char panel_event_sw_no;
static unsigned short panel_timer_count;


#define PANEL_SCAN_KO0_PORT	(PORTB.PODR.BIT.B0)
#define PANEL_SCAN_KO1_PORT	(PORTB.PODR.BIT.B1)
#define PANEL_SCAN_KO2_PORT	(PORTB.PODR.BIT.B3)
#define	PANEL_SCAN_KI_PORT	(PORT4.PIDR.BYTE)
#define	PANEL_SCAN_KI_MASK1	(0x1c /*01011100*/)
#define	PANEL_SCAN_KI_SHIFT1	(2)
#define	PANEL_SCAN_KI_MASK2	(0x40 /*01011100*/)
#define	PANEL_SCAN_KI_SHIFT2	(3)

#define	PANEL_CHAT_COUNT_INIT	(/*2*/20)
#define	PANEL_SCAN_KEEP_PUSH_TIME	(700)
#define	PANEL_SCAN_KEEP_PUSH_TIME2	(700)


//==========================================
//  panel_scan_initialize
//      plXL
//==========================================
void panel_scan_initialize(void)
{
extern void wait_10m(void);
unsigned short scan_d,scan_d2;

	//
	panel_scan_state = 0;
	panel_chat_count = 0;
	scan_d = panel_tmp_map = 0;
	//	tact sw map init
	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 1;
	nop();
	wait_10m();
	nop();
	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	panel_tmp_map = scan_d;

	PANEL_SCAN_KO0_PORT = 0;
	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 1;
	nop();
	wait_10m();
	nop();
	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	scan_d <<= 4;
	panel_tmp_map |= scan_d;

	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 0;
	PANEL_SCAN_KO2_PORT = 1;
	nop();
	wait_10m();
	nop();
	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	scan_d <<= 8;
	panel_tmp_map |= scan_d;

	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 1;

	panel_old_map = panel_new_map = panel_tmp_map;
	
	// ȂÑ`FbN 
	
	
	
	
}

//==========================================
//  panel_scan
//      plXL
//==========================================
void panel_scan(void)
{

	switch(panel_scan_state){
		case 0:	// KO0 pulse out(init)
			panel_scan_0();
			break;
		case 1:	// KO0 read & KO1 pulse out
			panel_scan_1();
			break;
		case 2:	// KO1 read & KO2 pulse out
			panel_scan_2();
			break;
		case 3:	// KO2 read & KO0 pulse out
			panel_scan_3();
			break;
		case 4:	// event check
			panel_scan_4();
			break;
		case 5:	// dial event check
			panel_scan_5();
			break;
	}
}

//==========================================
//  panel_scan_0
//      plXL  KO0 pulse out(init)
//==========================================
static void panel_scan_0(void)
{
	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 1;

	panel_scan_state = 1;
}

//==========================================
//  panel_scan_1
//      plXL  KO0 read & KO1 pulse out
//==========================================
static void panel_scan_1(void)
{
unsigned short scan_d,scan_d2;


	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	PANEL_SCAN_KO0_PORT = 0;
	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 1;
	nop();
	
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	panel_tmp_map &= ~0x000f;
	panel_tmp_map |= scan_d;
	
	panel_scan_state = 2;
}

//==========================================
//  panel_scan_2
//      plXL  KO1 read & KO2 pulse out
//==========================================
static void panel_scan_2(void)
{
unsigned short scan_d,scan_d2;


	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 0;
	PANEL_SCAN_KO2_PORT = 1;
	nop();
	
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	scan_d <<= 4;
	panel_tmp_map &= ~0x00f0;
	panel_tmp_map |= scan_d;
	
	panel_scan_state = 3;
}

//==========================================
//  panel_scan_3
//      plXL  KO2 read & KO0 pulse out
//==========================================
static void panel_scan_3(void)
{
unsigned short scan_d,scan_d2;


	scan_d = scan_d2 = PANEL_SCAN_KI_PORT;
	nop();
	PANEL_SCAN_KO2_PORT = 0;
	PANEL_SCAN_KO1_PORT = 0;
	PANEL_SCAN_KO0_PORT = 1;
	nop();
	
	scan_d &= PANEL_SCAN_KI_MASK1;
	scan_d >>= PANEL_SCAN_KI_SHIFT1;
	scan_d2 &= PANEL_SCAN_KI_MASK2;
	scan_d2 >>= PANEL_SCAN_KI_SHIFT2;
	scan_d |= scan_d2;
	scan_d <<= 8;
	panel_tmp_map &= ~0x0f00;
	panel_tmp_map |= scan_d;
	
	panel_scan_state = 4;
}

//==========================================
//  panel_scan_4
//      plXL  event check
//==========================================
static void panel_scan_4(void)
{
unsigned short event_map;

	panel_long_push_enter();
	
	if(panel_tmp_map != panel_new_map){
		panel_new_map = panel_tmp_map;
		panel_chat_count = PANEL_CHAT_COUNT_INIT;
	}else{
		if(panel_chat_count != 0){
			panel_chat_count --;
			if(panel_chat_count == 0){
			
				// check off
				event_map = (panel_old_map ^ panel_new_map)&(panel_old_map);
				if((event_map & 0x0001)!=0x0000){	// [RF_AMP]
					panel_off_enter(1);
				}
				if((event_map & 0x0002)!=0x0000){	// [SET]
					panel_off_enter(2);
				}
				if((event_map & 0x0004)!=0x0000){	// [MODE]
					panel_off_enter(3);
				}
				if((event_map & 0x0008)!=0x0000){	// [RIT]
					panel_off_enter(4);
				}
				if((event_map & 0x0010)!=0x0000){	// [MENU]
					panel_off_enter(5);
				}
				if((event_map & 0x0020)!=0x0000){	// [AGC]
					panel_off_enter(6);
				}
				if((event_map & 0x0040)!=0x0000){	// [BAND]
					panel_off_enter(7);
				}
				if((event_map & 0x0080)!=0x0000){	// [left]
					panel_off_enter(8);
				}
				if((event_map & 0x0100)!=0x0000){	// [ESC]
					panel_off_enter(9);
				}
				if((event_map & 0x0200)!=0x0000){	// [FILTER]
					panel_off_enter(10);
				}
				if((event_map & 0x0400)!=0x0000){	// [A/B]
					panel_off_enter(11);
				}
				if((event_map & 0x0800)!=0x0000){	// [right]
					panel_off_enter(12);
				}
				
				// check on
				event_map = (panel_old_map ^ panel_new_map)&(panel_new_map);
				if((event_map & 0x0001)!=0x0000){	// [RF_AMP]
//					sw_on_RfAmp();
					panel_on_enter(1);
				}
				if((event_map & 0x0002)!=0x0000){	// [SET]
					panel_on_enter(2);
				}
				if((event_map & 0x0004)!=0x0000){	// [MODE]
//					sw_on_Mode();
					panel_on_enter(3);
				}
				if((event_map & 0x0008)!=0x0000){	// [RIT]
//					sw_on_Rit();
					panel_on_enter(4);
				}
				if((event_map & 0x0010)!=0x0000){	// [MENU]
					panel_on_enter(5);
				}
				if((event_map & 0x0020)!=0x0000){	// [AGC]
//					sw_on_Agc();
					panel_on_enter(6);
				}
				if((event_map & 0x0040)!=0x0000){	// [BAND]
//					sw_on_Band();
					panel_on_enter(7);
				}
				if((event_map & 0x0080)!=0x0000){	// [left]
//					sw_on_FreqCursor_left();
					panel_on_enter(8);
				}
				if((event_map & 0x0100)!=0x0000){	// [ESC]
					panel_on_enter(9);
				}
				if((event_map & 0x0200)!=0x0000){	// [FILTER]
//					sw_on_Filter();
					panel_on_enter(10);
				}
				if((event_map & 0x0400)!=0x0000){	// [A/B]
//					sw_on_Vfo();
					panel_on_enter(11);
				}
				if((event_map & 0x0800)!=0x0000){	// [right]
//					sw_on_FreqCursor_right();
					panel_on_enter(12);
				}
				
				//
				panel_old_map = panel_new_map;
			}
		}
	}
	panel_scan_state = 5;
}

//==========================================
//  panel_scan_5
//      plXL  dial event check
//==========================================
static void panel_scan_5(void)
{
unsigned short dialval;
short dial_delta;
extern void sw_dial_AG(short delta);
extern void sw_dial_SQ(short delta);
extern void sw_dial_Dial(short delta);

	// Dial
//	dialval = dial_get_count_MyEnc1();
	dialval = dial_get_count_Enc();
	dial_delta = dialval - panel_dial_Dial_old;
	panel_dial_Dial_old = dialval;
	sw_dial_Dial(dial_delta);
	
	// AG
//	dialval = dial_get_count_Enc();
	dialval = dial_get_count_MyEnc1();
	dial_delta = dialval - panel_dial_AG_old;
	panel_dial_AG_old = dialval;
	sw_dial_AG(dial_delta);
	
	// SQ
	dialval = dial_get_count_MyEnc2();
	dial_delta = dialval - panel_dial_SQ_old;
	panel_dial_SQ_old = dialval;
	sw_dial_SQ(dial_delta);
	
	// for next
	panel_scan_state = 1;
}

//==========================================
//  panel_on_enter
//      plONCxgJn
//		in: no	SWԍ
//==========================================
static void panel_on_enter(unsigned char no)
{
	panel_event_sw_no = no;
	panel_timer_start(PANEL_SCAN_KEEP_PUSH_TIME);
}

//==========================================
//  panel_on_continue
//      pl@văX^[g
//		in: no	SWԍ
//==========================================
void panel_on_continue(unsigned char no)
{
	panel_event_sw_no = no;
	panel_timer_start(PANEL_SCAN_KEEP_PUSH_TIME2);
}

//==========================================
//  panel_off_enter
//      plOFFCxgJn
//		in: no	SWԍ
//==========================================
static void panel_off_enter(unsigned char no)
{
extern void sw_on_enter(unsigned char);

	if(panel_event_sw_no == no){
		if(panel_timer_flag != 0){	//~ł͂Ȃ
			panel_timer_stop();
			sw_on_enter(no);
		}
	}
}

//==========================================
//  panel_long_push_enter
//      plFJn
//==========================================
static void panel_long_push_enter(void) 
{
extern void sw_long_on_enter(unsigned char);

	if(panel_timer_flag == 0xff){	//F
		panel_timer_stop();
		sw_long_on_enter(panel_event_sw_no);
	}
}

//==========================================
//  panel_timer
//      1ms^C}[荞ݏ
//==========================================
void panel_timer(void)
{
extern unsigned char band_sw_wait_count;
	if(panel_timer_flag==1){	// when now count
		if(panel_timer_count!=0){
			panel_timer_count--;
			if(panel_timer_count==0){
				panel_timer_flag = 0xff;
			}
		}
	}
	if(band_sw_wait_count != 0){
		band_sw_wait_count--;
	}
}

//==========================================
//  panel_timer_start
//      panel timer start
//		in: wait count (1ms)
//==========================================
void panel_timer_start(unsigned short count)
{
	panel_timer_count = count;
	panel_timer_flag = 1;
}

//==========================================
//  panel_timer_stop
//      panel timer stop
//==========================================
void panel_timer_stop(void)
{
	panel_timer_count = 0;
	panel_timer_flag = 0;
}




void sw_test(unsigned char v, char *ptr)
{
return;
	lcd_locate(0,0,0);
	lcd_putchar('[');
	lcd_print(ptr);
	lcd_putchar(']');
	if(v==0){
		lcd_print("OFF    ");
	}else{
		lcd_print("ON     ");
	}
}



