/***********************************************************************/
/*                                                                     */
/*  FILE        :switch.c                                              */
/*  DESCRIPTION :SW operation proc                                     */
/*  CPU TYPE    :RX210                                                 */
/*                                                                     */
/*  last modefied : Nov 12, 2013                                       */
/*  for CQ-HF Panel   AOR  by KOBAYASHI Ikuo                           */
/*                                                                     */
/***********************************************************************/
                  
#include <machine.h>
#include "iodefine.h"
#include "display.h"
#include "vfo_param.h"

#ifdef __cplusplus
extern "C" {
#endif

void sw_on_enter(unsigned char no);
void sw_long_on_event(unsigned char no);

void sw_on_FreqCursor_left(void);
void sw_on_FreqCursor_right(void);
void sw_on_RfAmp(void);
void sw_on_Agc(void);
void sw_on_Filter(void);
void sw_on_Mode(void);
void sw_on_Rit(void);
void sw_on_Vfo(void);
void sw_on_Band(void);
void sw_on_RitClear(void);
void sw_on_VfoCopy(void);
void sw_on_Menu(void);
void sw_on_Set(void);
void sw_on_Esc(void);
void sw_on_MenuParam_keep(void);
void sw_on_MenuParam_cancel(void);
void sw_on_Set_MenuSel(void);
void sw_on_Set_MenuEdit(void);
void sw_dial_MenuEdit_inc(void);
void sw_dial_MenuEdit_dec(void);
void sw_dial_AG(short delta);
void sw_dial_SQ(short delta);
void sw_dial_Dial(short delta);
void sw_beep_OK(void);
void sw_beep_NG(void);





#ifdef __cplusplus
}
#endif

extern void backup_write_req(unsigned char v);

unsigned long MenuOldLong1;
unsigned short MenuOldShort1;

unsigned char MainMode;
unsigned char FreqCursorPos,RitCursorPos;
unsigned char RitInputFlag;
unsigned char MenuNo;
unsigned char MenuOldChar1,MenuOldChar2;
unsigned char BeepVol;
unsigned char IfFreqInputFlag,IfFreqCursorPos;
unsigned char band_sw_wait_count;

extern struct VfoParamStruct CurrentVfo,VfoArray[2];

extern long RitOffset;
extern unsigned long IFFreq;

extern unsigned short CWPitch;
extern short ShiftFreq;

extern unsigned char VfoNo;
extern unsigned char SQOpened;
//extern unsigned char FreqCursorPos;
extern unsigned char VolVal,SQVal;
extern unsigned char RFGain,MicGain;
extern unsigned char /*VoiceSQ_Kando,*/VoiceSQ_Delay,LevelSQ_Hysteresis;
extern unsigned char CurrentVoiceSQ_Kando,CurrentVoiceSQ_Delay,CurrentVoiceSQ_Enable;
extern unsigned char DeNoise,AutoNotch,NoiseBlanker;
extern unsigned char AfcVal;
extern unsigned char SemiBreakInHoldTime;
extern unsigned char AMModulationDepth,TXPower;
extern unsigned char TXMeterType;
extern unsigned char SuperHeteroFlag;
extern unsigned char TestModeFlag,TestToneFlag;
extern unsigned char IFShiftEnable;
extern unsigned char CWCarrier,CWSideTone;


static unsigned long f_d[] = {
	       1L,
	      10L,
	     100L,
	    1000L,
	   10000L,
	  100000L,
	 1000000L,
	10000000L
};

//==========================================
//  sw_on_enter
//      SW ONCxgJn
//		in: no	SWԍ
//==========================================
void sw_on_enter(unsigned char no)
{
	switch(no){
		case 1:	// [RF_AMP]
			sw_on_RfAmp();
			break;
		case 2:	// [SET]
			sw_on_Set();
			break;
		case 3:	// [MODE]
			sw_on_Mode();
			break;
		case 4:	// [RIT]
			sw_on_Rit();
			break;
		case 5:	// [MENU]
			sw_on_Menu();
			break;
		case 6:	// [AGC]
			sw_on_Agc();
			break;
		case 7:	// [BAND]
			sw_on_Band();
			break;
		case 8: // [left]
			sw_on_FreqCursor_left();
			break;
		case 9:	// [ESC]
			sw_on_Esc();
			break;
		case 10:	// [FILTER]
			sw_on_Filter();
			break;
		case 11:	// [A/B]
			sw_on_Vfo();
			break;
		case 12:	// [right]
			sw_on_FreqCursor_right();
			break;
	}
}

//==========================================
//  sw_long_on_enter
//      SW CxgJn
//		in: no	SWԍ
//==========================================
void sw_long_on_enter(unsigned char no)
{
extern void panel_on_continue(unsigned char no);
	switch(no){
		case 1:	// [RF_AMP]
			sw_on_RfAmp();
			break;
		case 2:	// [SET]
			sw_on_Set();
			break;
		case 3:	// [MODE]
			sw_on_Mode();
			break;
		case 4:	// [RIT]
			sw_on_RitClear();
			break;
		case 5:	// [MENU]
			sw_on_Menu();
			break;
		case 6:	// [AGC]
			sw_on_Agc();
			break;
		case 7:	// [BAND]
			sw_on_Band();
			break;
		case 8: // [left]
			sw_on_FreqCursor_left();
			panel_on_continue(no);
			break;
		case 9:	// [ESC]
			sw_on_Esc();
			break;
		case 10:	// [FILTER]
			sw_on_Filter();
			break;
		case 11:	// [A/B]
			sw_on_VfoCopy();
			break;
		case 12:	// [right]
			sw_on_FreqCursor_right();
			panel_on_continue(no);
			break;
	}
}




//==========================================
//  sw_on_FreqCursor_left
//		sw   [ < ]
//==========================================
void sw_on_FreqCursor_left(void)
{
unsigned char f;
extern unsigned char NowPopUp,NowCursorPopUp;
	f = 0;
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(RitInputFlag == 0){
				if(NowPopUp==1){
					return;
				}
				if(FreqCursorPos < 7){
					FreqCursorPos++;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(FreqCursorPos);
			}else{
				if(RitCursorPos < 3){
					RitCursorPos++;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(RitCursorPos);
			}
			if(f!=0){
				sw_beep_OK();
			}else{
				sw_beep_NG();
			}
			break;
		case MAIN_MODE_MENU_EDIT:
			if(IfFreqInputFlag != 0){
				if(IfFreqCursorPos < 7){
					IfFreqCursorPos++;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(IfFreqCursorPos);
				if(f!=0){
					sw_beep_OK();
				}else{
					sw_beep_NG();
				}
			}
			break;
	}
}

//==========================================
//  sw_on_FreqCursor_right
//		sw   [ > ]
//==========================================
void sw_on_FreqCursor_right(void)
{
unsigned char f;
extern unsigned char NowPopUp,NowCursorPopUp;
	f = 0;
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(RitInputFlag == 0){
				if(NowPopUp==1){
					return;
				}
				if(FreqCursorPos > 0){
					FreqCursorPos--;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(FreqCursorPos);
			}else{
				if(RitCursorPos > 0){
					RitCursorPos--;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(RitCursorPos);
			}
			if(f!=0){
				sw_beep_OK();
			}else{
				sw_beep_NG();
			}
			break;
		case MAIN_MODE_MENU_EDIT:
			if(IfFreqInputFlag != 0){
				if(IfFreqCursorPos > 0){
					IfFreqCursorPos--;
					f++;
				}
				lcd_disp_Cursor_popup();
				lcd_disp_FreqCursor(IfFreqCursorPos);
				if(f!=0){
					sw_beep_OK();
				}else{
					sw_beep_NG();
				}
			}
			break;
	}
}

//==========================================
//  sw_on_RfAmp
//		sw   [AMP]
//==========================================
void sw_on_RfAmp(void)
{
unsigned char v;
extern void dem_APA_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			v = CurrentVfo.Flag.BIT.RfAmp;
			v = (v == 0)? 1: 0;
			CurrentVfo.Flag.BIT.RfAmp = VfoArray[VfoNo].Flag.BIT.RfAmp = v;
			dem_APA_set();
			lcd_disp_RfAmp_icon(v);
			lcd_disp_RfAmp_popup(v);
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Agc
//		sw   [AGC]
//==========================================
void sw_on_Agc(void)
{
unsigned char v;
extern void dem_AGC_param_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			v = CurrentVfo.AGC;
			v++;
			if(v >= AGC_PARAM_ITEM_MAX){
				v = 0;
			}
			CurrentVfo.AGC = VfoArray[VfoNo].AGC = v;
			dem_AGC_param_set();
			if(v==0){
				IFShiftEnable = 0;
				lcd_disp_IFShift_icon(IFShiftEnable,ShiftFreq);
				dem_AIS_set();
			}
			lcd_disp_Agc_icon(v);
			lcd_disp_Agc_popup(v);
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Filter
//		sw   [FILTER]
//==========================================
void sw_on_Filter(void)
{
unsigned char v;
extern void dem_AIF_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(CurrentVfo.Mode == MODE_ITEM_CW){
				v = 4;
			}else if(CurrentVfo.Mode == MODE_ITEM_WFM){
				v = CurrentVfo.IFBandWidth;
				v++;
				if(v >= FILTER_ITEM_NOT_WFM){
					v = 0;
				}
			}else{
				v = CurrentVfo.IFBandWidth;
				v++;
				if(v >= FILTER_ITEM_MAX){
					v = FILTER_ITEM_NOT_WFM;
				}
			}
				CurrentVfo.IFBandWidth = VfoArray[VfoNo].IFBandWidth = v;
			dem_AIF_set();
			lcd_disp_Filter_icon(v);
			lcd_disp_Filter_popup(v);
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Mode
//		sw   [MODE]
//==========================================
void sw_on_Mode(void)
{
unsigned char v,b,t;
extern void dem_ARM_set(void);
extern void dem_AIF_set(void);
extern unsigned char vfo_Freq_to_Band(unsigned long freq, unsigned char *band);
extern unsigned char NowTxFlag;
	switch(MainMode){
		case MAIN_MODE_VFO:
			v = CurrentVfo.Mode;
			v++;
			if(v >= MODE_ITEM_MAX){
				if(vfo_Freq_to_Band(CurrentVfo.CurrentFreq,&b)){
					if(b==11){
						v = 0;
					}else{
						v = 1;
					}
				}else{
					v = 1;
				}
			}
			CurrentVfo.Mode = VfoArray[VfoNo].Mode = v;
			switch(v){
				case 0:	// WFM
					CurrentVfo.IFBandWidth = VfoArray[VfoNo].IFBandWidth = 1;	// 200k
					break;
				case 1:	// FM
					CurrentVfo.IFBandWidth = VfoArray[VfoNo].IFBandWidth = 1 + FILTER_ITEM_NOT_WFM;	// 15k
					break;
				case 2:	// AM
				case 3:	// SAM
					CurrentVfo.IFBandWidth = VfoArray[VfoNo].IFBandWidth = 2 + FILTER_ITEM_NOT_WFM;	// 6k
					break;
				case 4:	// LSB
				case 5:	// USB
					CurrentVfo.IFBandWidth = VfoArray[VfoNo].IFBandWidth = 3 + FILTER_ITEM_NOT_WFM;	// 3k
					break;
			}
			dem_ARM_set();
			dem_AIF_set();
			
			if(NowTxFlag==0){
				t = vfo_isAmatureFreq(CurrentVfo.CurrentFreq,v) ? 0: 2;
			}else{
				t = 1;
			}
			lcd_disp_TX_icon(t);
	
			if(v == MODE_ITEM_CW){
				lcd_disp_Filter_icon(FILTER_ITEM_MAX/*4*/);
			}else{
				lcd_disp_Filter_icon(CurrentVfo.IFBandWidth);
			}
			lcd_disp_Mode_icon(v);
			lcd_disp_Mode_popup(v);
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Rit
//		sw   [RIT]
//==========================================
void sw_on_Rit(void)
{
unsigned long freq;
unsigned char v;
extern unsigned char NowTxFlag;
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(NowTxFlag == 0){
				v = vfo_isAmatureFreq(CurrentVfo.CurrentFreq,CurrentVfo.Mode) ? 0: 2;
			}else{
				v = 1;
			}
			lcd_disp_TX_icon(v);
			if(RitInputFlag==0){
				RitInputFlag = 1;
				lcd_disp_Rit_icon(RitOffset);
				lcd_disp_RitOffset(RitOffset);
				lcd_disp_FreqCursor(RitCursorPos);
			}else{
				RitInputFlag = 0;
				lcd_disp_Rit_icon(RitOffset);
				freq = CurrentVfo.CurrentFreq;
				if(NowTxFlag == 0){
					freq += RitOffset;
				}
				lcd_disp_RxFreq(freq/*CurrentVfo.CurrentFreq*/);
				lcd_disp_FreqCursor(FreqCursorPos);
			}
			sw_beep_OK();
			break;
	}
}

//==========================================
//  sw_on_Vfo
//		sw   [A/B]
//==========================================
void sw_on_Vfo(void)
{
extern void dem_ARF_set(void);
extern void dem_ARM_set(void);
extern void dem_AIF_set(void);
extern void dem_AGC_param_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			RitInputFlag = 0;
			if(VfoNo == 0){	// when A
				VfoNo = 1;
			}else{	// when B
				VfoNo = 0;
			}
			CurrentVfo = VfoArray[VfoNo];
			dem_ARF_set();
			dem_ARM_set();
			dem_AIF_set();
			dem_AGC_param_set();
			lcd_disp_AllRefresh();
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Band
//		sw   [BAND]
//==========================================
void sw_on_Band(void)
{
unsigned char band,r;
extern unsigned char vfo_Freq_to_Band(unsigned long freq, unsigned char *band);
extern unsigned char vfo_Freq_to_NextBand(unsigned long freq);
extern void dem_ARF_set(void);
extern void dem_ARM_set(void);
extern void dem_AIF_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			RitInputFlag = 0;
			// ܂݂͌̎gL
			r = vfo_Freq_to_Band(CurrentVfo.CurrentFreq,&band);
			if(r){
				CurrentVfo.BandFreq[band] = CurrentVfo.CurrentFreq;
			}
			// ̃ohɃWv
			band = vfo_Freq_to_NextBand(CurrentVfo.CurrentFreq);
			if(band == 11){	// FM Broadband
				CurrentVfo.Mode = 0;	// WFM
				CurrentVfo.IFBandWidth = 1;	// 100k
			}else if(band <= 3){	// 7NHzȉ
				CurrentVfo.Mode = 4;	// LSB
				CurrentVfo.IFBandWidth = 3 + FILTER_ITEM_NOT_WFM;	// 3k
			}else{	// 10MHzȏ
				CurrentVfo.Mode = 5;	// USB
				CurrentVfo.IFBandWidth = 3 + FILTER_ITEM_NOT_WFM;	// 3k
			}
			CurrentVfo.CurrentFreq = CurrentVfo.BandFreq[band];
			//
			band_sw_wait_count = 10;
			VfoArray[VfoNo] = CurrentVfo;
			dem_ARF_set();
			dem_ARM_set();
			dem_AIF_set();
			lcd_disp_AllRefresh();
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_RitClear
//		sw   [RIT]
//==========================================
void sw_on_RitClear(void)
{
extern void dem_ARF_set(void);
	switch(MainMode){
		case MAIN_MODE_VFO:
			RitInputFlag = 0;
			RitOffset = 0L;
			dem_ARF_set();
			lcd_disp_AllRefresh();
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_VfoCopy
//		sw   [A/B]
//==========================================
void sw_on_VfoCopy(void)
{
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(VfoNo == 0){	// when A
				VfoArray[1] = CurrentVfo;
			}else{	// when B
				VfoArray[0] = CurrentVfo;
			}
			lcd_disp_AllRefresh();
			sw_beep_OK();
			backup_write_req(1);
			break;
	}
}

//==========================================
//  sw_on_Menu
//		sw   [MENU]
//==========================================
void sw_on_Menu(void)
{
	switch(MainMode){
		case MAIN_MODE_MENU_EDIT:
			MainMode = MAIN_MODE_MENU_SEL;
			//ɍ
		case MAIN_MODE_MENU_SEL:
			sw_on_Esc();
			break;
		case MAIN_MODE_VFO:
			MainMode = MAIN_MODE_MENU_SEL;
			MenuNo = 0;
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			lcd_disp_SetCursor_MenuNo();
			sw_beep_OK();
			break;
	}
}

//==========================================
//  sw_on_Set
//		sw   [SET]
//==========================================
void sw_on_Set(void)
{
extern void vfo_param_init(void);
extern void lcd_disp_MenuParam_Reset_Complete(void);
extern void backup_EraceAll_req(void);
	switch(MainMode){
		case MAIN_MODE_MENU_SEL:
			sw_on_MenuParam_keep();
			MainMode = MAIN_MODE_MENU_EDIT;
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			lcd_disp_SetCursor_MenuParam();
			sw_beep_OK();
			break;
		case MAIN_MODE_MENU_EDIT:
			MainMode = MAIN_MODE_MENU_SEL;
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			if(MenuNo==MENU_PAGE_RESET){	// when Reset
				backup_EraseAll_req();
				vfo_param_init();
				lcd_disp_MenuParam_Reset_Complete();
			}
			lcd_disp_SetCursor_MenuNo();
			sw_beep_OK();
			backup_write_req(2);
			break;
	}
}

//==========================================
//  sw_on_Esc
//		sw   [ESC]
//==========================================
void sw_on_Esc(void)
{
	switch(MainMode){
		case MAIN_MODE_MENU_SEL:
			MainMode = MAIN_MODE_VFO;
			lcd_disp_AllRefresh();
			sw_beep_OK();
			break;
		case MAIN_MODE_MENU_EDIT:
			sw_on_MenuParam_cancel();
			MainMode = MAIN_MODE_MENU_SEL;
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			lcd_disp_SetCursor_MenuNo();
			sw_beep_OK();
			backup_write_req(2);
			break;
	}
}

//==========================================
//  sw_on_MenuParam_keep
//		sw   Menu Edit  param keep
//==========================================
void sw_on_MenuParam_keep(void)
{
	IfFreqInputFlag = 0;
	switch(MenuNo){
		case 0:	// SQ Hysteresis
			MenuOldChar1 = LevelSQ_Hysteresis;
			break;
		case 1:	// Voice SQ on/off
			MenuOldChar1 = CurrentVoiceSQ_Enable;
			break;
		case 2:	// VSQ Sensitivity
			MenuOldChar1 = CurrentVoiceSQ_Kando;
			break;
		case 3:	// VSQ_Delay
			MenuOldChar1 = CurrentVoiceSQ_Delay;
			break;
		case 4:	// DeNoise
			MenuOldChar1 = DeNoise;
			break;
		case 5:	// AutoNotch
			MenuOldChar1 = AutoNotch;
			break;
		case 6:	// NoiseBlanker
			MenuOldChar1 = NoiseBlanker;
			break;
		case 7:	// CWPitch
			MenuOldShort1 = CWPitch;
			break;
		case 8:	// CWSideToneLevel
			MenuOldShort1 = CWSideTone;
			break;
		case 9:	// CWCarrierLevel
			MenuOldShort1 = CWCarrier;
			break;
		case 10:	// SemiBreakIn HoldTime
			MenuOldChar1 = SemiBreakInHoldTime;
			break;
		case 11:	// AFC
			MenuOldChar1 = AfcVal;
			break;
		case 12:	// IF Shift
//			MenuOldShort1 = ShiftFreq;
			MenuOldChar1 = IFShiftEnable;
			break;
		case 13:	// MIC Gain
			MenuOldChar1 = MicGain;
			break;
		case 14:	// AMModulation
			MenuOldChar1 = AMModulationDepth;
			break;
		case 15:	// TXPower
			MenuOldChar1 = TXPower;
			break;
		case 16:	// TXMeter
			MenuOldChar1 = TXMeterType;
			break;
		case 17:	// SuperHetero
			MenuOldChar1 = SuperHeteroFlag;
			break;
		case 18:	// IF Freq
			MenuOldLong1 = IFFreq;
			IfFreqInputFlag = 1;
			break;
		case 19:	// TestMode
			MenuOldChar1 = TestModeFlag;
			break;
		case 20:	// TestTone
			MenuOldChar1 = TestToneFlag;
			break;
		case 21:	// BeepVol
			MenuOldChar1 = BeepVol;
			break;
		case 22:	// Reset
			break;
		case 23:	// Version
			break;
	}
}

//==========================================
//  sw_on_MenuParam_cancel
//		sw   Menu Edit  param cancel
//==========================================
void sw_on_MenuParam_cancel(void)
{
extern void dem_ARF_set(void);
extern void dem_AVO_set(void);
	switch(MenuNo){
		case 0:	// SQ Hysteresis
			LevelSQ_Hysteresis = MenuOldChar1;
			dem_ALH_AHL_set();
			break;
		case 1:	// Voice SQ on/off
			CurrentVoiceSQ_Enable = MenuOldChar1;
			dem_AVS_set();
			break;
		case 2:	// VSQ Sensitivity
			CurrentVoiceSQ_Kando = MenuOldChar1;
			dem_AVP_set();
			break;
		case 3:	// VSQ Delay
			CurrentVoiceSQ_Delay = MenuOldChar1;
			dem_AVP_set();
			break;
		case 4:	// DeNoise
			DeNoise = MenuOldChar1;
			dem_ADN_set();
			break;
		case 5:	// AutoNotch
			AutoNotch = MenuOldChar1;
			dem_AAN_set();
			break;
		case 6:	// NoiseBlanker
			NoiseBlanker = MenuOldChar1;
			dem_ABN_set();
			break;
		case 7:	// CWPitch
			CWPitch = MenuOldShort1;
			dem_ACP_set();
			break;
		case 8:	// CWSideToneLevel
			CWSideTone = MenuOldShort1;
			dem_AVO_set();
			break;
		case 9:	// CWCarrierLevel
			CWCarrier = MenuOldShort1;
			dem_AAM_set();
			break;
		case 10:	// SemiBreakIn HoldTime
			SemiBreakInHoldTime = MenuOldChar1;
			dem_ACH_set();
			break;
		case 11:	// AFC
			AfcVal = MenuOldChar1;
			dem_AFN_set();
			break;
		case 12:	// IF Shift
//			ShiftFreq = MenuOldShort1;
			IFShiftEnable = MenuOldChar1;
			dem_AIS_set();
			break;
		case 13:	// MIC Gain
			MicGain = MenuOldChar1;
			dem_AMG_set();
			break;
		case 14:	// AMModulationDepth
			AMModulationDepth = MenuOldChar1;
			dem_AAM_set();
			break;
		case 15:	// TXPower
			TXPower = MenuOldChar1;
			dem_AOP_set();
			break;
		case 16:	// TXMeterType
			TXMeterType = MenuOldChar1;
			break;
		case 17:	// SuperHetero
			SuperHeteroFlag = MenuOldChar1;
			dem_AHD_set();
			dem_ARF_set();
			break;
		case 18:	// IF Freq
			IFFreq = MenuOldLong1;
			IfFreqInputFlag = 0;
			dem_ARF_set();
			break;
		case 19:	// TestMode
			TestModeFlag = MenuOldChar1;
			dem_ADX_set();
			break;
		case 20:	// TestTone
			TestToneFlag = MenuOldChar1;
			dem_ATM_set();
			break;
		case 21:	// BeepVol
			BeepVol = MenuOldChar1;
			dem_ABV_set();
			break;
		case 22:	// Reset
			break;
		case 23:	// Version
			break;
	}
}

//==========================================
//  sw_dial_MenuEdit_inc
//		sw   Menu Edit  inc
//==========================================
void sw_dial_MenuEdit_inc(void)
{
extern void dem_ARF_set(void);
extern void dem_AVO_set(void);
	switch(MenuNo){
		case 0:	// SQ Hysteresis
			LevelSQ_Hysteresis ++;
			if(LevelSQ_Hysteresis > SQ_HYSTERESIS_MAX){
				LevelSQ_Hysteresis = SQ_HYSTERESIS_MAX;
			}
			dem_ALH_AHL_set();
			break;
		case 1:	// Voice SQ on/off
			CurrentVoiceSQ_Enable = (CurrentVoiceSQ_Enable != 0) ? 0: 1;
			dem_AVS_set();
			break;
		case 2:	// VoiceSQ Sensitivity
			CurrentVoiceSQ_Kando ++;
			if(CurrentVoiceSQ_Kando > VSQ_KANDO_MAX){
				CurrentVoiceSQ_Kando = VSQ_KANDO_MAX;
			}
			dem_AVP_set();
			break;
		case 3:	// VoiceSQ Delay
			CurrentVoiceSQ_Delay ++;
			if(CurrentVoiceSQ_Delay > VSQ_DELAY_MAX){
				CurrentVoiceSQ_Delay = VSQ_DELAY_MAX;
			}
			dem_AVP_set();
			break;
		case 4:	// DeNoise
			DeNoise++;
			if(DeNoise > DENOISE_MAX){
				DeNoise = DENOISE_MAX;
			}
			dem_ADN_set();
			break;
		case 5:	// AutoNotch
			AutoNotch++;
			if(AutoNotch > AUTO_NOTCH_MAX){
				AutoNotch = AUTO_NOTCH_MAX;
			}
			dem_AAN_set();
			break;
		case 6:	// NoiseBlanker
			NoiseBlanker = (NoiseBlanker != 0) ? 0: 1;
			dem_ABN_set();
			break;
		case 7:	// CWPitch
			CWPitch += 50;
			if(CWPitch > CW_PITCH_MAX){
				CWPitch = CW_PITCH_MAX;
			}
			dem_ACP_set();
			break;
		case 8:	// CWSideToneLevel
			CWSideTone++;
			if(CWSideTone > VOL_MAX){
				CWSideTone = VOL_MAX;
			}
			dem_AVO_set();
			break;
		case 9:	// CWCarrierLevel
			CWCarrier++;
			if(CWCarrier > AM_MODULATION_MAX){
				CWCarrier = AM_MODULATION_MAX;
			}
			dem_AAM_set();
			break;
		case 10:	// SemiBreakIn HoldTime
			SemiBreakInHoldTime ++;
			if(SemiBreakInHoldTime > SEMI_BREAK_IN_MAX){
				SemiBreakInHoldTime = SEMI_BREAK_IN_MAX;
			}
			dem_ACH_set();
			break;
		case 11:	// AFC
			AfcVal = (AfcVal != 0) ? 0: 1;
			dem_AFN_set();
			break;
		case 12:	// IF Shift
//			ShiftFreq += 50;
//			if(ShiftFreq > SHIFT_FREQ_MAX){
//				ShiftFreq = SHIFT_FREQ_MAX;
//			}
//			dem_AIS_set();
			IFShiftEnable = (IFShiftEnable != 0) ? 0: 1;
			dem_AIS_set();
			break;
		case 13:	// MIC Gain
			MicGain++;
			if(MicGain > MIC_GAIN_MAX){
				MicGain = MIC_GAIN_MAX;
			}
			dem_AMG_set();
			break;
		case 14:	// AMModulationDepth
			AMModulationDepth ++;
			if(AMModulationDepth > AM_MODULATION_MAX){
				AMModulationDepth = AM_MODULATION_MAX;
			}
			dem_AAM_set();
			break;
		case 15:	// TXPower
			TXPower ++;
			if(TXPower > TX_POWER_MAX){
				TXPower = TX_POWER_MAX;
			}
			dem_AOP_set();
			break;
		case 16:	// TXMeterType
			TXMeterType++;
			if(TXMeterType > TX_METER_TYPE_MAX){
				TXMeterType = TX_METER_TYPE_MAX;
			}
			break;
		case 17:	// SuperHetero
			SuperHeteroFlag = (SuperHeteroFlag != 0) ? 0: 1;
			dem_AHD_set();
			dem_ARF_set();
			break;
		case 18:	// IF Freq
			IFFreq += f_d[IfFreqCursorPos];
			if(IFFreq > IF_FREQ_HIGH_LIMIT){
				IFFreq = IF_FREQ_HIGH_LIMIT;
			}
			dem_ARF_set();
			break;
		case 19:	// TestMode
			TestModeFlag = (TestModeFlag != 0) ? 0: 1;
			dem_ADX_set();
			break;
		case 20:	// TestTone
			TestToneFlag = (TestToneFlag != 0) ? 0: 1;
			dem_ATM_set();
			break;
		case 21:	// BeepVol
			BeepVol ++;
			if(BeepVol > BEEP_VOL_MAX){
				BeepVol = BEEP_VOL_MAX;
			}
			dem_ABV_set();
			break;
		case 22:	// Reset
			break;
		case 23:	// Version
			break;
	}
}

//==========================================
//  sw_dial_MenuEdit_dec
//		sw   Menu Edit  dec
//==========================================
void sw_dial_MenuEdit_dec(void)
{
extern void dem_ARF_set(void);
extern void dem_AVO_set(void);
	switch(MenuNo){
		case 0:	// SQ Hysteresis
			if(LevelSQ_Hysteresis != 0){
				LevelSQ_Hysteresis --;
			}
			dem_ALH_AHL_set();
			break;
		case 1:	// Voice SQ on/off
			CurrentVoiceSQ_Enable = (CurrentVoiceSQ_Enable != 0) ? 0: 1;
			dem_AVS_set();
			break;
		case 2:	// VoiceSQ Sensitivity
			if(CurrentVoiceSQ_Kando != 0){
				CurrentVoiceSQ_Kando --;
			}
			dem_AVP_set();
			break;
		case 3:	// VoiceSQ Delay
			if(CurrentVoiceSQ_Delay != 0){
				CurrentVoiceSQ_Delay --;
			}
			dem_AVP_set();
			break;
		case 4:	// DeNoise
			if(DeNoise != 0){
				DeNoise --;
			}
			dem_ADN_set();
			break;
		case 5:	// AutoNotch
			if(AutoNotch != 0){
				AutoNotch--;
			}
			dem_AAN_set();
			break;
		case 6:	// NoiseBlanker
			NoiseBlanker = (NoiseBlanker != 0) ? 0: 1;
			dem_ABN_set();
			break;
		case 7:	// CWPitch
			CWPitch -= 50;
			if(CWPitch < CW_PITCH_MIN){
				CWPitch = CW_PITCH_MIN;
			}
			dem_ACP_set();
			break;
		case 8:	// CWSideToneLevel
			if(CWSideTone != 0){
				CWSideTone--;
			}
			dem_AVO_set();
			break;
		case 9:	// CWCarrierLevel
			if(CWCarrier != 0){
				CWCarrier--;
			}
			dem_AAM_set();
			break;
		case 10:	// SemiBreakIn HoldTime
			if(SemiBreakInHoldTime != 0){
				SemiBreakInHoldTime --;
			}
			dem_ACH_set();
			break;
		case 11:	// AFC
			AfcVal = (AfcVal != 0) ? 0: 1;
			dem_AFN_set();
			break;
		case 12:	// IF Shift
//			ShiftFreq -= 50;
//			if(ShiftFreq < SHIFT_FREQ_MIN){
//				ShiftFreq = SHIFT_FREQ_MIN;
//			}
//			dem_AIS_set();
			IFShiftEnable = (IFShiftEnable != 0) ? 0: 1;
			dem_AIS_set();
			break;
		case 13:	// MIC Gain
			if(MicGain != 0){
				MicGain--;
			}
			dem_AMG_set();
			break;
		case 14:	// AMModulationDepth
			if(AMModulationDepth != 0){
				AMModulationDepth --;
			}
			dem_AAM_set();
			break;
		case 15:	// TXPower
			if(TXPower != 0){
				TXPower --;
			}
			dem_AOP_set();
			break;
		case 16:	// TXMeterType
			if(TXMeterType != 0){
				TXMeterType --;
			}
			break;
		case 17:	// SuperHetero
			SuperHeteroFlag = (SuperHeteroFlag != 0) ? 0: 1;
			dem_AHD_set();
			dem_ARF_set();
			break;
		case 18:	// IF Freq
			IFFreq -= f_d[IfFreqCursorPos];
			if(IFFreq < IF_FREQ_LOW_LIMIT){
				IFFreq = IF_FREQ_LOW_LIMIT;
			}
			if(IFFreq > IF_FREQ_HIGH_LIMIT){
				IFFreq = IF_FREQ_LOW_LIMIT;
			}
			dem_ARF_set();
			break;
		case 19:	// TestMode
			TestModeFlag = (TestModeFlag != 0) ? 0: 1;
			dem_ADX_set();
			break;
		case 20:	// TestTone
			TestToneFlag = (TestToneFlag != 0) ? 0: 1;
			dem_ATM_set();
			break;
		case 21:	// BeepVol
			if(BeepVol != 0){
				BeepVol --;
			}
			dem_ABV_set();
			break;
		case 22:	// RFeset
			break;
		case 23:	// Version
			break;
	}
}


//==========================================
//  sw_dial_AG
//		sw   dial[AG]
//==========================================
void sw_dial_AG(short delta)
{
extern void dem_AVO_set(void);
	if(delta==0){
		return;
	}
	
	if(delta > 0){	// when +
		while(delta > 0){
			VolVal ++;
			if(VolVal > VOL_MAX){
				VolVal = VOL_MAX;
			}
			delta--;
		}
	}else{	// when -
		while(delta < 0){
			if(VolVal > 0){
				VolVal--;
			}
			delta++;
		}
	}
	lcd_disp_AG_popup(VolVal);
	dem_AVO_set();
	backup_write_req(0);
}

//==========================================
//  sw_dial_SQ
//		sw   dial[SQ]
//==========================================
void sw_dial_SQ(short delta)
{
extern void dem_ASQ_set(void);
extern void dem_AGL_set(void);
	if(delta==0){
		return;
	}
	if(delta > 0){	// when +
		while(delta > 0){
			if(CurrentVfo.AGC == 0){
				RFGain ++;
				if(RFGain > RFGAIN_MAX){
					RFGain = RFGAIN_MAX;
				}
			}else if(IFShiftEnable != 0){
				ShiftFreq += 50;
				if(ShiftFreq > SHIFT_FREQ_MAX){
					ShiftFreq = SHIFT_FREQ_MAX;
				}
			}else{
				SQVal ++;
				if(SQVal > SQ_MAX){
					SQVal = SQ_MAX;
				}
			}
			delta--;
		}
	}else{	// when -
		while(delta < 0){
			if(CurrentVfo.AGC == 0){
				if(RFGain > 0){
					RFGain--;
				}
			}else if(IFShiftEnable != 0){
				ShiftFreq -= 50;
				if(ShiftFreq < SHIFT_FREQ_MIN){
					ShiftFreq = SHIFT_FREQ_MIN;
				}
			}else{
				if(SQVal > 0){
					SQVal--;
				}
			}
			delta++;
		}
	}
	if(CurrentVfo.AGC == 0){
		lcd_disp_RFGain_popup(RFGain);
		dem_AGL_set();
	}else if(IFShiftEnable != 0){
		lcd_disp_IFShift_icon(IFShiftEnable,ShiftFreq);
		lcd_disp_IFShift_popup(ShiftFreq);
		dem_AIS_set();
	}else{
		lcd_disp_SQ_popup(SQVal);
		dem_ASQ_set();
	}
	backup_write_req(0);
}

//==========================================
//  sw_dial_Dial
//		sw   dial[dial]
//==========================================
void sw_dial_Dial(short delta)
{
unsigned long freq;
long fl;
//static unsigned long f_d[] = {
//	       1L,
//	      10L,
//	     100L,
//	    1000L,
//	   10000L,
//	  100000L,
//	 1000000L,
//	10000000L
//};
extern void dem_ARF_set(void);
extern void lcd_disp_Dial_popup(unsigned long val,unsigned char mode);
extern unsigned char NowTxFlag;

	if(delta==0){
		return;
	}
	
	switch(MainMode){
		case MAIN_MODE_VFO:
			if(NowTxFlag != 0){
				return;
			}
			if(RitInputFlag==0){
				freq = CurrentVfo.CurrentFreq;
				if(delta > 0){	//when +
					while(delta > 0){
						freq += f_d[FreqCursorPos];
						if(freq > FREQ_HIGH_LIMIT){
							freq = FREQ_HIGH_LIMIT;
						}
						delta--;
					}
				}else{	// when -
					while(delta < 0){
						freq -= f_d[FreqCursorPos];
						if(freq > FREQ_HIGH_LIMIT){
							freq = FREQ_LOW_LIMIT;
						}
						if(freq < FREQ_LOW_LIMIT){
							freq = FREQ_LOW_LIMIT;
						}
						delta++;
					}
				}
				CurrentVfo.CurrentFreq = VfoArray[VfoNo].CurrentFreq = freq;
				dem_ARF_set();
				lcd_disp_Dial_popup(CurrentVfo.CurrentFreq,CurrentVfo.Mode);
			}else{
				fl = RitOffset;
				if(delta > 0){	// when +
					while(delta > 0){
						fl += f_d[RitCursorPos];
						if(fl > RIT_HIGH_LIMIT){
							fl = RIT_HIGH_LIMIT;
						}
						delta--;
					}
				}else{	// when -
					while(delta < 0){
						fl -= f_d[RitCursorPos];
						if(fl < RIT_LOW_LIMIT){
							fl = RIT_LOW_LIMIT;
						}delta++;
					}
				}
				RitOffset = fl;
				dem_ARF_set();
				lcd_disp_Rit_popup(RitOffset);
			}
			backup_write_req(0);
			break;
		case MAIN_MODE_MENU_SEL:
			if(delta > 0){
				while(delta > 0){
					MenuNo ++;
					if(MenuNo >= MENU_PAGE_MAX){
						MenuNo = 0;
					}
					delta--;
				}
				
			}else{
				while(delta < 0){
					if(MenuNo==0){
						MenuNo = MENU_PAGE_MAX -1;
					}else{
						MenuNo--;
					}
					delta++;
				}
			}
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			lcd_disp_SetCursor_MenuNo();
			break;
		case MAIN_MODE_MENU_EDIT:
			if(delta > 0){
				while(delta > 0){
					sw_dial_MenuEdit_inc();
					delta--;
				}
			}else{
				while(delta < 0){
					sw_dial_MenuEdit_dec();
					delta++;
				}
			}
			lcd_disp_MenuTitle(MenuNo);
			lcd_disp_MenuParam(MenuNo);
			lcd_disp_SetCursor_MenuParam();
			break;
		default:
			delta = 0;
			break;
	}
}

//==========================================
//  sw_beep_OK
//		sw   OK beep
//==========================================
void sw_beep_OK(void)
{
extern void dem_AKB_set(unsigned char val);
	if(BeepVol != 0){
		dem_AKB_set(0);
	}
}

//==========================================
//  sw_beep_NG
//		sw   NG beep
//==========================================
void sw_beep_NG(void)
{
extern void dem_AKB_set(unsigned char val);
	if(BeepVol != 0){
		dem_AKB_set(1);
	}
}

